############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
"""
Provides a simple interface to the ftp client module.
"""

from pyGlobus.util import GlobusException
from pyGlobus import utilc
from pyGlobus.ftpControl import Parallelism, TcpBuffer, Layout
import ftpClientc
from pyGlobus.util import Buffer
from pyGlobus.util import Abstime
from pyGlobus.util import Off_t
import threading, sys


#########################################################################
# Module classes.
#########################################################################
class FtpClientException(GlobusException):
    """
    A simple subclass of GlobusException.
    
    Used for errors in the ftp client  module. It defines no new methods.
    
    **Extends:**
     
         *pyGlobus.util.GlobusException*
    """
    pass

class HandleAttrException(GlobusException):
    """
    A simple subclass of GlobusException.
    
    Used for errors in the ftp client handle attr module. It defines
    no new methods.
    
    **Extends:**
     
         *pyGlobus.util.GlobusException*
    """
    pass

class OperationAttrException(GlobusException):
    """
    A simple subclass of GlobusException.
    
    Used for errors in the ftp client operation attr module. It defines
    no new methods.
    
    **Extends:**
     
         *pyGlobus.util.GlobusException*
    """
    pass

class RestartMarkerException(GlobusException):
    """
    A simple subclass of GlobusException.
    
    Used for errors in the ftp client restart marker module. It defines
    no new methods.
    
    **Extends:**
     
         *pyGlobus.util.GlobusException*
    """
    pass

class PluginException(GlobusException):
    """
    A simple subclass of GlobusException.
    
    Used for errors in the Plugin module. It defines
    no new methods.
    
    **Extends:**
     
         *pyGlobus.util.GlobusException*
    """
    pass

class BaseFtpClient:
    """Parent Class for the FtpClient and EasyFtpClient classes"""
    
    def __init__(self):
        raise RuntimeError, "Class can not be initialized"
    
    def mkdir(self, *argslist):
        raise NotImplementedError, "method mkdir has not been implemented in child class"

    def delete(self, *argslist):
        raise NotImplementedError, "delete method has not been implemented  in child class"

    def rmdir(self,*argslist):
        raise NotImplementedError, "rmdir method has not been implemented  in child class"

    def list(self, *argslist):
        raise NotImplementedError, "list method has not been implemented  in child class"

    def verbose_list(self, *argslist):
        raise NotImplementedError, "verbose_list method has not been implemented  in child class"

    def move(self, *argslist):
        raise NotImplementedError, "move method has not been implemented  in child class"

    def get(self, *argslist):
        raise NotImplementedError, "get method has not been implemented  in child class"

    def partial_get(self, *argslist):
        raise NotImplementedError, "partial_get method has not been implemented  in child class"

    def put(self, *argslist):
        raise NotImplementedError, "put method has not been implemented  in child class"

    def partial_put(self, *argslist):
        raise NotImplementedError, "partial_put method has not been implemented  in child class"

    def third_party_transfer(self, *argslist):
        raise NotImplementedError, "third_party_transfer method has not been implemented  in child class"
    
    def partial_third_party_transfer(self, *argslist):
        raise NotImplementedError, "partial_third_party_transfer method has not been implemented  in child class"                           
    def exists(self, *argslist):
        raise NotImplementedError, "exists method has not been implemented  in child class"
    
    def modification_time(self, *argslist):
        raise NotImplementedError," modification_time method has not been implemented  in child class"
    
    def size(self, *argslist):
        raise NotImplementedError,"size method has not been implemented  in child class"


    def cache_url_state(self, url):
        """
              Cache the connection to the FTP server.
              
              When the url is cached, the library will not close the connection after completing
              an operation.

              **Arguments:**
              
              - *url* The URL of the server to cache.

              **Raises:**

              A FtpClientException is thrown if unable to cache the url state.
              """
        
        ret, err = ftpClientc.handle_cache_url_state(self._handle, url)
        if not ret == 0:
            ex = FtpClientException(err)
            raise ex
        return

    def flush_url_state(self, url):
        """
              Remove the url from the connection cache.
              
              If an idle connection to the server exists, it will be closed.
              
              **Arguments:**
              
              - *url* The URL to remove from the cache.

              **Raises:**

              A FtpClientException is thrown if unable to flush the url state.
              """
        
        ret, err = ftpClientc.handle_flush_url_state(self._handle, url)
        if not ret == 0:
            ex = FtpClientException(err)
            raise ex
        return

    def set_user_data(self, data):
        """
              Associate some data with the handle.
              
              Access to this data is not synchronized.

              **Arguments:**

              - *data* The user data to set.

              **Raises:**

              An FtpClientException is thrown if unable to set the data.
              """
        
        ret, err = ftpClientc.handle_set_user_pointer(self._handle, data)
        if not ret == 0:
            ex = FtpClientException(err)
            raise ex
        return

    def get_user_data(self):
        """
              Returns the user data associated with the handle.
              
              Access to this data is not synchronized.

              **Returns:**

              The data set with the set_user_data method.
              
              **Raises:**

              An FtpClientException is thrown if unable to get the data.
              """
        
        ret, data = ftpClientc.handle_get_user_pointer(self._handle, data)
        if not ret == 0:
            ex = FtpClientException(data)
            raise ex
        return data

    def abort(self):
        """
              Abort the current operation.
              
              **Raises:**

              A FtpClientException is thrown if an error occurs while aborting
              the current operation.
              """
        
        ret, err = ftpClientc.py_abort(self._handle)
        
        if not ret == 0:
            ex = FtpClientException(err)
            raise ex
        return

    def add_plugin(self, plugin):
        """
              Add a plugin to an FTP client handle.
              
              This function adds a plugin to an FTP client handle after it has been
              created. Plugins may be added to an ftp client handle whenever an
              operation is not in progress. The plugin will be appended to the list of
              plugins present in the handle, and will be invoked during any subsequent
              operations processed with this handle.

              Only one instance of a particular plugin may be added to a particular handle.
              
              **Arguments**

              *plugin* A Plugin class.

              **Raises**

              A FtpClientException is thrown if unable to add the plugin.
              """
        
        ret, err = ftpClientc.handle_add_plugin(self._handle, plugin._handle)
        if not ret == 0:
            ex = FtpClientException(err)
            raise ex
        return

    def remove_plugin(self, plugin):
        """
              Remove a plugin to an FTP client handle.
              
              This function removes a plugin from an FTP client handle after it has been
              created. Plugins may be removed from an ftp client handle whenever an
              operation is not in progress. The plugin will be removed from the list of
              plugins, and will not be used during any subsequent operations processed
              with this handle.
              
              **Arguments**

              *plugin* A Plugin class.

              **Raises**

              A FtpClientException is thrown if unable to remove the plugin.
              """
        
        ret, err = ftpClientc.handle_remove_plugin(self._handle, plugin._handle)
        if not ret == 0:
            ex = FtpClientException(err)
            raise ex
        return
    
    def get_handle(self):
        """
              Return the underlying ftp client handle.
              
              **XXX** This handle is only valid as long as this object exists.  When
              the object is destroyed, the memory this handle points to is freed.
              
              **Returns:**

              A string containing the pointer.
              """
        return self._handle

class FtpClient(BaseFtpClient):
    """
    A class to wrap the ftp client functions.

    It provides blocking and non-blocking transfers and access to the other ftp
    operations.
    """
    
    def __init__(self, handleAttr, handle=None):
        """
              Constructs an instance.
              
              Create a new FtpClient instance. This will init the ftp client module and
              create a new ftp client handle if none is passed in.
              
              **Arguments:**

              - *handleAttr* A HandleAttr object with the attributes set for
              this object.

              - *handle* If a SWIG'ized pointer to a globus ftp client handle object
              is passed in, create a new instance around this handle. The class
              then owns the underlying pointer, and will delete it when the instance
              is destroyed.
              
              **Raises:**

              A FtpClientException is thrown if unable to init the module or create
              the underlying handle.
              """
        
        self._activated = 0
        self._cbList = []
        ret = utilc.globus_module_activate(ftpClientc.get_module())
        if not ret == 0:
            ex = FtpClientAttrException("Unable to initialize the module")
            raise ex
        self._activated = 1
        if handle == None:
            ret, handle = ftpClientc.handle_init(handleAttr.get_handle())
            if not ret == 0:
                ex = FtpClientException(handle)
                raise ex
        self._handle = handle
        return

    def __del__(self):
        """
              Destroy an instance.
              
              Free's any remaining callbacks, destroyes the underlying handle, and deactivates
              the module.
              
              **Raises:**
              
              A FtpClientException is thrown if unable to destroy the handle or deactivate
              the module.
              """
    
        for cb in self._cbList:
            self.free_callback(cb)
        ret, err = ftpClientc.handle_destroy(self._handle)
        if not ret == 0:
            ex = FtpClientException(err)
            raise ex
        ret = utilc.globus_module_deactivate(ftpClientc.get_module())
        if not ret == 0:
            ex = FtpClientException("Unable to deactivate the module")
            raise ex

        return
    
    def delete(self, url, attr, callback, arg):
        """
              Delete a file from a ftp server.
              
              The callback will be called with the final status of the delete.

              **Arguments:**

              - *url* The URL to delete.
              
              - *attr* An OperationAttr object containing the attributes for this
              delete.

              - *callback* The function to be called when the delete is completed. It
              has the following signature:

              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.

              - *arg* A user argument to be returned when the callback is executed.

              **Returns:**

              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
            
              **Raises:**

              A FtpClientException is thrown if unable to initi ate the delete.
              """
        
        ret, cbHandle = ftpClientc.py_delete(self._handle, url,
                                         attr.get_handle(),
                                         callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def mkdir(self, url, attr, callback, arg):
        """
              Make a directory on a ftp server.
              
              The callback will be called with the final status of the mkdir.
              
              **Arguments:**

              - *url* The URL to make.
              
              - *attr* An OperationAttr object containing the attributes for this
              mkdir.
              
              - *callback* The function to be called when the mkdir is completed. It
              has the following signature:
              
              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.

              - *arg* A user argument to be returned when the callback is executed.

              **Returns:**

              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
              
              **Raises:**

              A FtpClientException is thrown if unable to initi ate the mkdir.
              """
        
        ret, cbHandle = ftpClientc.py_mkdir(self._handle, url,
                                           attr.get_handle(),
                                           callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def rmdir(self, url, attr, callback, arg):
        """
              Remove a directory on a ftp server.
              
              The callback will be called with the final status of the rmdir.
              
              **Arguments:**
              
              - *url* The URL to remove.
              
              - *attr* An OperationAttr object containing the attributes for this
              rmdir.

              - *callback* The function to be called when the rmdir is completed. It
              has the following signature:
              
              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.
              
              - *arg* A user argument to be returned when the callback is executed.

              **Returns:**
              
              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
            
              **Raises:**

              A FtpClientException is thrown if unable to initi ate the rmdir.
              """
        
        ret, cbHandle = ftpClientc.py_rmdir(self._handle, url,
                                           attr.get_handle(),
                                           callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def list(self, url, attr, callback, arg):
        """
              Get a file listing from an ftp server.
              
              This method starts a 'NLIST' transfer from an ftp server.  The client may begin
              calling register_read to retrieve the listing, after this method returns. When all
              of the data associated with the listing is returned, and all of the data callbacks
              executed, the callback will be called with the final status of the list.

              **Arguments:**

              - *url* The url to list.
              
              - *attr* An OperationAttr object containing the attributes
              for the listing.
              
              - *callback* The function to be called when the list is completed. It
              has the following signature:
              
              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.
              
              - *arg* A user argument to be returned when the callback is executed.
              
              **Returns:**
              
              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
              
              **Raises:**

              A FtpClientException is thrown if unable to initiate the list.
              """
        
        ret, cbHandle = ftpClientc.py_list(self._handle, url,
                                          attr.get_handle(),
                                          callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def verbose_list(self, url, attr, callback, arg):
        """
              Get a file listing from an ftp server.
              
              This method starts a ' LIST' transfer from an ftp server.  The client may begin
              calling register_read to retrieve the listing, after this method returns. When all
              of the data associated with the listing is returned, and all of the data callbacks
              executed, the callback will be called with the final status of the list.
              
              **Arguments:**

              - *url* The url to list.
              
              - *attr* An OperationAttr object containing the attributes
              for the listing.

              - *callback* The function to be called when the list is completed. It
              has the following signature:
              
              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.

              - *arg* A user argument to be returned when the callback is executed.

              **Returns:**

              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
              
              **Raises:**

              A FtpClientException is thrown if unable to initiate the list.
              """
        
        ret, cbHandle = ftpClientc.verbose_list(self._handle, url,
                                               attr.get_handle(),
                                               callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def move(self, srcUrl, destUrl, attr, callback, arg):
        """
              Move a file on a ftp server.
              
              The complete callback will be called when the move is completed.
              
              **Arguments:**

              - *srcUrl* The url to move.
              
              - *destUrl* The url to move to.

              - *attr* An OperationAttr object containing the attributes
              for the listing.
              
              - *callback* The function to be called when the move is completed. It
              has the following signature:
              
              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.
              
              - *arg* A user argument to be returned when the callback is executed.

              **Returns:**
              
              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
            
              **Raises:**

              A FtpClientException is thrown if unable to initiate the move.
              """
        
        ret, cbHandle = ftpClientc.move(self._handle, srcUrl,
                                       destUrl, attr.get_handle(),
                                       callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def get(self, url, callback, arg,attr = None, marker = None):
        """
              Get a file from an ftp server.
              
              After calling this method, register_read can be called to begin retrieving
              the data.  When all of the data associated with this URL is retrieved, and all
              of the data callbacks have completed, or if the get request is aborted, the
              complete callback will be called with the final status of the get.
        
              **Arguments:**

              - *url* The url to download.

              - *attr* An OperationAttr object containing the attributes for
              this get.
            
              - *marker* A RestartMarker object.

              - *callback* The function to be called when the get is completed. It
              has the following signature:

              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.

              - *arg* A user argument to be returned when the callback is executed.
              
              **Returns:**

              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
            
              **Raises:**

              A FtpClientException is thrown if unable to initiate the get.
              """
        
        if attr == None:
            attr = OperationAttr()

        if marker == None:
            marker = RestartMarker()

        ret, cbHandle = ftpClientc.get(self._handle, url, attr.get_handle(),
                                      marker.get_handle(), callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def partial_get(self, url, attr, marker, offset, endOffset, callback, arg):
        """
              Get part of a file from a ftp server.
              
              After calling this method, register_read can be called to begin retrieving
              the data.  When all of the data associated with this URL is retrieved, and all
              of the data callbacks have completed, or if the get request is aborted, the
              complete callback will be called with the final status of the get.
              
              **Arguments:**

              - *url* The url to download.
              
              - *attr* An OperationAttr object containing the attributes for
              this get.

              - *marker* A RestartMarker object.
              
              - *offset* An int or long offset to begin the transfer at.

              - *endOffset* An int or long offset to end the transfer at.
              
              - *callback* The function to be called when the partial get is completed. It
              has the following signature:
            
              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.

              - *arg* A user argument to be returned when the callback is executed.
              
              **Returns:**

              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
            
              **Raises:**

              A FtpClientException is thrown if unable to initiate the partial get.
              """
        
        ret, cbHandle = ftpClientc.partial_get(self._handle, url, attr.get_handle(),
                                              marker.get_handle(), offset,
                                              endOffset, callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def extended_get(self, url, eret_alg_str, callback,
                     arg,attr = None, marker = None):
        """
              Get a file from an ftp server with server-side processing..

              After calling this method, register_read can be called to begin retrieving
              the data.  When all of the data associated with this URL is retrieved, and all
              of the data callbacks have completed, or if the get request is aborted, the
              complete callback will be called with the final status of the get.
              
              This method differs from the get method by allowing the user to invoke
              server-side data processing algorithms. GridFTP servers may support
              support algorithms for data reduction or other customized data storage
              requirements. There is no client-side verification done on the algorithm
              string provided by the user. If the server does not understand the
              requested algorithm, the transfer will fail.

              **Arguments:**

              - *url* The url to download.

              - *attr* An OperationAttr object containing the attributes for
              this get.

              - *marker* A RestartMarker object.
            
              - *eret_alg_str*  The ERET algorithm string. This string contains
              information needed to invoke a server-specific data reduction
              algorithm on the file being retrieved.
            
              - *callback* The function to be called when the get is completed. It
              has the following signature:
            
              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.
              
              - *arg* A user argument to be returned when the callback is executed.

              **Returns:**

              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
            
              **Raises:**

              A FtpClientException is thrown if unable to initiate the extended_get.
              """

        if attr == None:
            attr = OperationAttr()

        if marker == None:
            marker = RestartMarker()

            ret, cbHandle = ftpClientc.extended_get(self._handle, url,
                                                    attr.get_handle(),
                                                    marker.get_handle(),
                                                    eret_alg_str,
                                                    callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle
    
    def put(self, url, callback, arg, attr = None, marker = None):
        """
              Store a file on an ftp server.
              
              After calling this method, register_write can be called to begin writing
              the data.  When all of the data associated with this URL is sent, and all
              of the data callbacks have completed, or if the put request is aborted, the
              complete callback will be called with the final status of the put.

              **Arguments:**

              - *url* The url to store the data to.

              - *attr* An OperationAttr object containing the attributes for
              this put.

              - *marker* A RestartMarker object.
              
              - *callback* The function to be called when the put is completed. It
              has the following signature:

              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.
              
              - *arg* A user argument to be returned when the callback is executed.

              **Returns:**

              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
              
              **Raises:**
              
              A FtpClientException is thrown if unable to initiate the put.
              """

        if attr == None:               
	    attr = OperationAttr()

        if marker == None:
            marker = RestartMarker()


        ret, cbHandle = ftpClientc.put(self._handle, url, attr.get_handle(),
                                      marker.get_handle(), callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def partial_put(self, url, attr, marker, offset, endOffset, callback, arg):
        """
              Put part of a file onto a ftp server.
              
              After calling this method, register_write can be called to begin writing
              the data.  When all of the data associated with this URL is sent, and all
              of the data callbacks have completed, or if the partial_put request is aborted, the
              complete callback will be called with the final status of the partial_put.

              **Arguments:**
              
              - *url* The url to download.
            
              - *attr* An OperationAttr object containing the attributes for
              this partial_put.

              - *marker* A RestartMarker object.

              - *offset* An int or long offset to begin the transfer at.

              - *endOffset* An int or long offset to end the transfer at.

              - *callback* The function to be called when the partial put is completed. It
              has the following signature:

              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.
              
              - *arg* A user argument to be returned when the callback is executed.

              **Returns:**

              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
              
              **Raises:**
        
              A FtpClientException is thrown if unable to initiate the partial put.
              """
        
        ret, cbHandle = ftpClientc.partial_put(self._handle, url, attr.get_handle(),
                                              marker.get_handle(), offset,
                                              endOffset, callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle
    
    def extended_put(self, url, esto_alg_str,callback,
                     arg, attr = None, marker = None):
        """
              Store a file on an ftp server with server-side processing.
              
              After calling this method, register_write can be called to begin writing
              the data.  When all of the data associated with this URL is sent, and all
              of the data callbacks have completed, or if the put request is aborted, the
              complete callback will be called with the final status of the put.

              This method differs from the put method by allowing the user to invoke
              server-side data processing algorithms. GridFTP servers may support
              algorithms for data reduction or other customized data storage requirements.
              There is no client-side verification done on the alogirhtm string provided by
              the user. if the server does not understand * the requested algorithm, the transfer
              will fail.

              **Arguments:**
              
              - *url* The url to store the data to.
            
              - *attr* An OperationAttr object containing the attributes for
              this put.

              - *marker* A RestartMarker object.

              - *esto_alg_str* The algorith string. This string contains
              information needed to invoke a server-specific data reduction
              algorithm on the file being put.
            
              - *callback* The function to be called when the put is completed. It
              has the following signature:

              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.
              
              - *arg* A user argument to be returned when the callback is executed.

              **Returns:**

              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
              
              **Raises:**

              A FtpClientException is thrown if unable to initiate the put.
              """

        if attr == None:               
	    attr = OperationAttr()

        if marker == None:
            marker = RestartMarker()


        ret, cbHandle = ftpClientc.put(self._handle, url, attr.get_handle(),
                                      marker.get_handle(), callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle
    
    def third_party_transfer(self, srcUrl, destUrl, callback, arg,
                             srcAttr = None, destAttr = None, marker = None):
 
        """
              Transfer a file between two ftp servers.
              
              When the transfer is complete, the complete callback will be called with the
              final status of the transfer.
              
              **Arguments:**

              - *srcUrl* The url to transfer.

              - *srcAttr* An OperationAttr object containing the attributes
              for the source.

              - *destUrl* The url to store the data to.

              - *destAttr* An OperationAttr object containing the attributes
              for the destination.

              - *marker* A RestartMarker object.
              
              - *callback* The function to be called when the transfer is completed. It
              has the following signature:
              
              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.

              - *arg* A user argument to be returned when the callback is executed.

              **Returns:**

              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
            
              **Raises:**

              A FtpClientException is thrown if unable to initiate the transfer.
              """

        if srcAttr == None:
            srcAttr = OperationAttr()

        if destAttr == None:
            destAttr = OperationAttr()

        if marker == None:
            marker = RestartMarker()

        print 
        ret, cbHandle = ftpClientc.third_party_transfer(self._handle,
                                                       srcUrl,
                                                       srcAttr.get_handle(),
                                                       destUrl,
                                                       destAttr.get_handle(),
                                                       marker.get_handle(),
                                                       callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def partial_third_party_transfer(self, srcUrl, srcAttr, destUrl, destAttr,
                                     marker, offset, endOffset, callback, arg):
        """
              Transfer part of a file between two ftp servers.
              
              When the transfer is complete, the complete callback will be called with the
              final status of the transfer.

              **Arguments:**

              - *srcUrl* The url to transfer.

              - *srcAttr* An OperationAttr object containing the attributes
              for the source.

              - *destUrl* The url to store the data to.
              
              - *destAttr* An OperationAttr object containing the attributes
              for the destination.
            
              - *marker* A RestartMarker object.

              - *offset* An int or long offset to start the transfer from.

              - *endOffset* An int or long offset to end the transfer at.

              - *callback* The function to be called when the transfer is completed. It
              has the following signature:

              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.

              - *arg* A user argument to be returned when the callback is executed.

              **Returns:**

              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
            
              **Raises:**

              A FtpClientException is thrown if unable to initiate the transfer.
              """
        
        ret, cbHandle = ftpClientc.partial_third_party_transfer(self._handle,
                                                               srcUrl,
                                                               srcAttr.get_handle(),
                                                               destUrl,
                                                               destAttr.get_handle(),
                                                               marker.get_handle(),
                                                               offset, endOffset,
                                                               callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def extended_third_party_transfer(self, srcUrl, eret_alg_str,
                                      destUrl, esto_alg_str,
                                      callback, arg, srcAttr = None,
                                      destAttr = None, marker = None):
 
        """
              Transfer a file between two ftp servers with server-side processing.
              
              When the transfer is complete, the complete callback will be called with the
              final status of the transfer.
              
              This method differs from the third_party_transfer method by allowing the user
              to invoke server-side data processing algorithms. GridFTP servers may
              support algorithms for data reduction or other customized data storage
              requirements. There is no client-side verification done on the alogirhtm
              string provided by the user. if the server does not understand
              the requested algorithm, the transfer will fail.
              
              **Arguments:**

              - *srcUrl* The url to transfer.

              - *srcAttr* An OperationAttr object containing the attributes
              for the source.

              - *eret_alg_str* The algorithm string for the source server. This
              string contains information needed to invoke a server-specific
              data reduction algorithm on the file being retrieved.
            
              - *destUrl* The url to store the data to.

              - *esto_alg_str* The algorithm string for the dest server. This string contains
              information needed to invoke a server-specific data reduction
              algorithm on the file being placed.
            
              - *destAttr* An OperationAttr object containing the attributes
              for the destination.

              - *marker* A RestartMarker object.

              - *callback* The function to be called when the transfer is completed. It
              has the following signature:

              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.

              - *arg* A user argument to be returned when the callback is executed.
              
              **Returns:**

              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
              
              **Raises:**

              A FtpClientException is thrown if unable to initiate the transfer.
              """

        if srcAttr == None:
            srcAttr = OperationAttr()

        if destAttr == None:
            destAttr = OperationAttr()

        if marker == None:
            marker = RestartMarker()

        print 
        ret, cbHandle = ftpClientc.extended_third_party_transfer(self._handle,
                                                                 srcUrl,
                                                                 srcAttr.get_handle(),
                                                                 eret_alg_str,
                                                                 destUrl,
                                                                 destAttr.get_handle(),
                                                                 esto_alg_str,
                                                                 marker.get_handle(),
                                                                 callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle
    
    def register_read(self, buffer, callback, arg):
        """
              Register a data buffer to handle part of the ftp data transfer.
              
              This can only be called after get or partial get has been called.
              
              **Arguments:**

              - *buffer* A pyGlobus.util.Buffer.Buffer object.. This hides the
              internal globus_byte_t data buffer.
              
              - *callback* The function to be called after the data has been
              read. It has the following signature:
              'function(arg, handle, buffer, bufHandle, bufLen, offset, eof, error)'
              where arg is the user argument passed into this call, handle is the
              underlying ftp_client handle, buffer is a Python buffer object, bufHandle
              is a SWIG'ized pointer to the underlying globus_byte_t array, bufLen
              is the length of the underlying globus_byte_t array, offset is the offset
              into the file this dta block contains, eof is a bool that is true if
              the end of the data transfer has been reached, and error is a string
              containing either 'NONE' or an error message.

              - *arg* A user argument to be passed through to the callback.

              **Returns:**
              
              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
            
              **Raises:**

              A FtpClientException is thrown if unable to initiate the read.
              """
        
        ret, cbHandle = ftpClientc.register_read(self._handle,
                                                buffer.get_handle(),
                                                buffer.get_size(),
                                                callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def register_write(self, buffer, size, offset, eof, callback, arg):
        """
              Register a data buffer to handle part of the ftp data transfer.
              
              This can only be called after put or partial put has been called.

              **Arguments:**

              - *buffer* A pyGlobus.util.Buffer.Buffer object.. This hides the
              internal globus_byte_t data buffer.
            
              - *size* The length of the buffer.

              - *offset* The offset of the buffer to be written. In extended block
              mode the data doesn't need to be sent in order.

              - *eof* True if this buffer contains the end of the data.
            
              - *callback* The function to be called after the data has been
              written. It has the following signature:
              'function(arg, handle, buffer, bufHandle, bufLen, offset, eof, error)'
              where arg is the user argument passed into this call, handle is the
              underlying ftp_client handle, buffer is a Python buffer object, bufHandle
              is a SWIG'ized pointer to the underlying globus_byte_t array, bufLen
              is the length of the underlying globus_byte_t array, offset is the offset
              into the file this dta block contains, eof is a bool that is true if
              the end of the data transfer has been reached, and error is a string
              containing either 'NONE' or an error message.
              
              - *arg* A user argument to be passed through to the callback.
              
              **Returns:**

              A SWIG'ized pointer to the callback handle is returned. After the callback
              completes, this may be free'd with the free_callback method, or it will be
              free'd when the instance is destroyed.
            
              **Raises:**

              A FtpClientException is thrown if unable to initiate the write.
              """
        
        ret, cbHandle = ftpClientc.register_write(self._handle, buffer,
                                                 size, offset, eof,
                                                 callback, arg)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    
    def free_callback(self, cbHandle):
        """
              Free a callback.
              
              Free's the underlying memory allocated for a callback structure.
        
              **Arguments:**

              - *cbHandle* A SWIG'ized pointer to a callback structure.
              """
        
        ftpClientc.free_callback(cbHandle)
        indx = self._cbList.index(cbHandle)
        del self._cbList[indx]
        return

    
    def size(self, url, callback, arg,attr =  None):
        """
              Retrieves the size of a file on a ftp server
              
              When the size is retrieved the complete callback is called
              
              **Arguments**
              
              - * url, the location of the file

              - * attr, An OperationAttr object, if none is given then
              the default OperationAttr is created and used
              
              - *callback* The function to be called when the file size has been retrieved. It
              has the following signature:

              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.

              - *arg* A user argument to be returned when the callback is executed.
              
              **Returns**
              An util.Off_t object, 

              WARNING the object underlying globus_off_t pointer will not be filled with any
              meaningful data until the callback function has been called

              **Raises**

              An FtpClientException if the size cannont be retrieved

              """

        if attr == None:
            attr = OperationAttr()

        ret, size, cbHandle = ftpClientc.size(self._handle, url, attr.get_handle(),
                                        callback, arg)

        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex

        
        new_size = Off_t(size)
        self._cbList.append(cbHandle)
        return new_size
        
    def modification_time(self, url, callback, arg,attr = None):
        """
              Retrieves the size of a file on a ftp server

              When the modification time is retrieved the complete callback is called
              
              **Arguments**

              - * url, the location of the file
              
              - * attr, An OperationAttr object, if none is given then
              the default OperationAttr is created and used
              
              - *callback* The function to be called when the modification time of the url
              has been retrieved. It has the following signature:
            
              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.

              - *arg* A user argument to be returned when the callback is executed.
              **Returns**
              An util.Abstime object.
              
              WARNING the Abstime object's underlying timeval struct will not be filled with any
              meaningful data until the callback function has been called

              **Raises**
              An FtpClientException if the modification time cannot be retrieved

              """

        if attr == None:
            attr = OperationAttr()
            
        ret, cbHandle, mtime = ftpClientc.modification_time(self._handle, url, attr.get_handle(),
                                                  callback, arg)
        if not ret == 0:
             ex = FtpClientException(cbHandle)
             raise ex

        
        self._cbList.append(cbHandle)
        t = Abstime(mtime)
        return t
        
    def exists(self, url, callback, arg, attr = None):
        """
              Determines whether a file exists on a ftp server

              When the existence has been determined  the complete callback is called
              
              **Arguments**

              - * url, the location of the file

              - *callback* The function to be called when the modification time of the url
              has been retrieved. It has the following signature:
            
              'function(arg, handle, error)' where arg is the argument passed into this
              call, handle is a SWIG'ized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.

              - *arg* A user argument to be returned when the callback is executed.
              
              - * attr, An OperationAttr object, if none is given then
              the default OperationAttr is created and used

              **Returns**
              a boolean indicating whether the file exists
              
              **Raises**
              An FtpClientException if the existence cannot be determined
              
              """
        
        if attr == None:
            attr = OperationAttr()

            
        ret, cbHandle = ftpClientc.exists(self._handle, url, attr.get_handle(),
                                          callback, arg)

        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex

        self._cbList.append(cbHandle)
        return
    
class EasyFtpClient(BaseFtpClient):
    """
    A class to wrap the ftp client functions.

    Each operation in this class blocks until the operation has completed

    There is currently no way to abort an operation

    If you must abort an operation use the FtpClient class
    """
    
    def __init__(self, handleAttr = None, handle=None):
        """
              Constructs an instance.
              
              Create a new FtpClient instance. This will init the ftp client module and
              create a new ftp client handle if none is passed in.

              **Arguments:**

              - *handleAttr* A HandleAttr object with the attributes set for
              this object.
            
              - *handle* If a SWIG'ized pointer to a globus ftp client handle object
              is passed in, create a new instance around this handle. The class
              then owns the underlying pointer, and will delete it when the instance
              is destroyed.
              
              **Raises:**

              A FtpClientException is thrown if unable to init the module or create
              the underlying handle.
              """
        
        self._activated = 0
        self._condV = threading.Condition(threading.Lock())
        
        
        ret = utilc.globus_module_activate(ftpClientc.get_module())
        if not ret == 0:
            ex = FtpClientAttrException("Unable to initialize the module")
            raise ex
        self._activated = 1
        
        if handleAttr == None:
            handleAttr = HandleAttr()
            
        
        if handle == None:
            ret, handle = ftpClientc.handle_init(handleAttr.get_handle())
            if not ret == 0:
                ex = FtpClientException(handle)
                raise ex
        self._handle = handle
        return

    def __del__(self):
        """
              Destroy an instance.
              
              Free's any remaining callbacks, destroyes the underlying handle, and deactivates
              the module.
              
              **Raises:**

              A FtpClientException is thrown if unable to destroy the handle or deactivate
              the module.
              """
    
        ret, err = ftpClientc.handle_destroy(self._handle)
        if not ret == 0:
            ex = FtpClientException(err)
            raise ex
        ret = utilc.globus_module_deactivate(ftpClientc.get_module())
        if not ret == 0:
            ex = FtpClientException("Unable to deactivate the module")
            raise ex

        return

    def done_cb_exists(self, arglist, handle, error):    
        """
              This callback is for the exists method
              It is called after the existence check has returned from the ftp Server
              """
        
        if error != "NONE":
            print "Error in EasyFtpClient.done_cb_exists ", error
            arglist[1].append(error)

        cv = arglist[0]
        cv.acquire()
        cv.notify()
        cv.release()
        
        return
        
    def done_cb(self, cv, handle, error):
        """
              Callback that has the signature
              function(arg, handle, error) where arg is the argument passed into this
              call, handle is a SWIGized pointer to a ftp client handle object, and error
              is a string containing 'NONE', or an error message.
              
              This function handles these types of callbacks in the easyFtpClient class
              """
        
        if error != "NONE":
            print error
           
        cv.acquire()
        cv.notify()
        cv.release()
        
        return

    def read_cb(self, fd, handle, buffer, bufHandle, bufLen, offset, eof, error):
        """
              This is a callback function that is for the register_read method
              It is called when a the buffer is full with data that has arrived to the handle
              It is used as a data callback for the *get methods
              """
        
        fd.seek(offset)      
        fd.write(buffer)
        if not eof:
            try:
                ret,cbHandle = ftpClientc.register_read(self._handle,self._buffer.get_handle(),
                                                        self._buffer.get_size(),
                                                        self.read_cb, fd)
                if not ret == 0:
                    ex = FtpClientException(cbHandle)
                    raise ex
                
                if self._cb_handle != None:
                    ftpClientc.free_callback(self._cb_handle)    
                self._cb_handle = cbHandle
                
            except Exception, ex:
                print "Exception", ex
                fd.close()
                raise
         
        return

    def write_cb(self, fd, handle, buffer, bufHandle, bufLen, offset, eof, error):
        """
              This is a callback function for the register_write method.
              It is called when the handle has recieved the buffer
              It is used as a data callback for the *put methods
              """
        print "in write_cb"
        print "offset: ", offset
        if not eof:
            self._buffer = fd.read(bufLen)    
            try:
                ret,cbHandle = ftpClientc.register_write (self._handle, self._buffer, len(self._buffer),
                                                    offset + bufLen, self._buffer == "",self.write_cb, fd)
                if not ret == 0:
                    ex = FtpClientException(cbHandle)
                    raise ex
            
                if self._cb_handle != None:
                    ftpClientc.free_callback(self._cb_handle)
                self._cb_handle = cbHandle

            except GlobusException, ex:
                print "Exception in EasyFtpClient.write_cb", ex
                fd.close()
                raise ex
       
        return    

    def write_cb_partial(self, args, handle, buffer, bufHandle, bufLen, offset, eof, error):
        """
              This is a callback function for register writes that need to keep track of offsets
              It is used in the partial_put method
              """
        
        if not eof:
            
            fd, endOffset = args
            if offset + bufLen == endOffset:
                self._buffer = fd.read(0)
            elif endOffset - offset < bufLen:
                self._buffer = fd.read(endOffset - offset)
            else:
                self._buffer = fd.read(bufLen)

            try:
                ret,cbHandle = ftpClientc.register_write (self._handle, self._buffer, len(self._buffer),
                                                    offset + bufLen, self._buffer == "",self.write_cb_partial,(fd, endOffset))

                if not ret == 0:
                    ex = FtpClientException(cbHandle)
                    raise ex
            
                if self._cb_handle != None:
                    ftpClientc.free_callback(self._cb_handle)
                self._cb_handle = cbHandle

            except GlobusException, ex:
                print "Exception in EasyFtpClient.write_cb", ex
                fd.close()
                raise ex
            
    def small_read_cb(self, arg, handle, buffer, bufHandle, bufLen, offset, eof, error):
        """
              This function is for doing small reads such as a verbose_list.
              The result is stored in a string, instead of being written to a file
              """

        self._result += str(buffer)
    
        if not eof:
            try:
                ret, cbHandle = ftpClientc.register_read(self._handle,self._buffer.get_handle(),
                                                         len(self._buffer), self.small_read_cb, 0)

                if not ret == 0:
                   ex = FtpClientException(cbHandle)
                   raise ex
                    
                if self._cb_handle != None:
                    ftpClientc.free_callback(self._cb_handle)
                self._cb_handle = cbHandle

            except Exception, ex:
                print "Exception", ex
                raise ex
        return

    def delete(self, url, attr = None):
        """
              Delete a file from a ftp server.
              
              This method blocks until the file has been deleted
              
              **Arguments:**

              - *url* The URL to delete.

              - *attr* An OperationAttr object containing the attributes for this
              delete.
              
              **Returns**
              Nothing is returned
              
              **Raises:**

              A FtpClientException is thrown if unable to initi ate the delete.
              """
        
        if attr == None:
            attr = OperationAttr()

        self._condV.acquire()
        ret, cbHandle = ftpClientc.py_delete(self._handle, url,
                                         attr.get_handle(),
                                         self.done_cb,self._condV )

        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        
        self._condV.wait()
        self._condV.release()
        ftpClientc.free_callback(cbHandle)

    def mkdir(self, url, attr= None):
        """
              Make a directory on a ftp server.
              
              The callback will be called with the final status of the mkdir.

              **Arguments:**

              - *url* The URL to make.

              - *attr* An OperationAttr object containing the attributes for this
              mkdir.
              
              **Returns**
              Nothing is returned
              
              **Raises:**

              A FtpClientException is thrown if unable to initi ate the mkdir.
              """

        if attr == None:
            attr = OperationAttr()

        self._condV.acquire()
        ret, cbHandle = ftpClientc.py_mkdir(self._handle, url,
                                           attr.get_handle(),
                                           self.done_cb, self._condV)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        
        
        self._condV.wait()
        self._condV.release()
        ftpClientc.free_callback(cbHandle)

    def rmdir(self, url, attr= None):
        """
              Remove a directory on a ftp server.
              
              The callback will be called with the final status of the rmdir.
              
              **Arguments:**

              - *url* The URL to remove.

              - *attr* An OperationAttr object containing the attributes for this
              rmdir.
              
              **Returns**
              Nothing is returned
              
              **Raises:**

              A FtpClientException is thrown if unable to initi ate the rmdir.
              """
        
        if attr == None:
            attr = OperationAttr()
            
        self._condV.acquire()
        ret, cbHandle = ftpClientc.py_rmdir(self._handle, url,
                                           attr.get_handle(),
                                           self.done_cb, self._condV)
        
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        
        
        self._condV.wait()
        self._condV.release()
        ftpClientc.free_callback(cbHandle)

    def list(self, url, attr = None):
        """
              Get a file listing from an ftp server.
              
              This method starts a 'NLIST' transfer from an ftp server.  The client may begin
              calling register_read to retrieve the listing, after this method returns. When all
              of the data associated with the listing is returned, and all of the data callbacks
              executed, the callback will be called with the final status of the list.

              **Arguments:**

              - *url* The url to list.

              - *attr* An OperationAttr object containing the attributes
              for the listing.

              **Returns**
              A string of the 'NLIST' transfer is returned
           
              **Raises:**
              
              A FtpClientException is thrown if unable to initiate the list.
              """

        if attr == None:
            attr = OperationAttr()
            
        self._condV.acquire()    
        ret, cbHandle = ftpClientc.py_list(self._handle, url,
                                          attr.get_handle(),
                                          self.done_cb, self._condV)
        
        
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex

        self._buffer = Buffer(1024 *64)
        self._result = ''
        self._cb_handle = None
        ret,handle = ftpClientc.register_read(self._handle,self._buffer.get_handle(),
                                              self._buffer.get_size(), self.small_read_cb, 0)

        if not ret == 0:
            ex = FtpClientException(handle)
            raise ex
        
        self._condV.wait()
        self._condV.release()
        ftpClientc.free_callback(cbHandle)
        return self._result
    
    def verbose_list(self, url, attr = None):
        """
              Get a file listing from an ftp server.
              
              This method blocks until the list of the url is retrieved
              
              This method starts a ' LIST' transfer from an ftp server.  The client may begin
              calling register_read to retrieve the listing, after this method returns. When all
              of the data associated with the listing is returned, and all of the data callbacks
              executed, the callback will be called with the final status of the list.

              **Arguments:**

              - *url* The url to list.

              - *attr* An OperationAttr object containing the attributes
              for the listing.

              **Returns**
              A string of the 'LIST' transfer is returned
            
              **Raises:**
              
              A FtpClientException is thrown if unable to initiate the list.
              """
                
        if attr == None:
            attr = OperationAttr()

        self._condV.acquire()
        ret, cbHandle = ftpClientc.verbose_list(self._handle, url,
                                               attr.get_handle(),
                                               self.done_cb, self._condV)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex

        self._buffer = Buffer(1024 *64)
        self._result = ''
        self._cb_handle = None
        ret,handle = ftpClientc.register_read(self._handle,self._buffer.get_handle(),
                                              self._buffer.get_size(), self.small_read_cb, 0)

        if not ret == 0:
            ex = FtpClientException(handle)
            raise ex
        
        
        self._condV.wait()
        self._condV.release()
        ftpClientc.free_callback(cbHandle)
        return self._result

    
    
    
    
    def move(self, srcUrl, destUrl, attr=None):
        """
              Move a file on a ftp server.
              
              The complete callback will be called when the move is completed.

              **Arguments:**

              - *srcUrl* The url to move.

              - *destUrl* The url to move to.

              - *attr* An OperationAttr object containing the attributes
              for the listing.
            
              ** Returns**
              Nothing is returned
            
              **Raises:**

              A FtpClientException is thrown if unable to initiate the move.
              """
        
        if attr == None:
            attr = OperationAttr()
            
        self._condV.acquire()
        ret, cbHandle = ftpClientc.move(self._handle, srcUrl,
                                       destUrl, attr.get_handle(),
                                       self.done_cb, self._condV )
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        
        self._condV.wait()
        self._condV.release()
        ftpClientc.free_callback(cbHandle)

   
    def get(self,dest_file, url, attr = None, marker = None):
        """
              Get a file from an ftp server.

              After calling this method, register_read can be called to begin retrieving
              the data.  When all of the data associated with this URL is retrieved, and all
              of the data callbacks have completed, or if the get request is aborted, the
              complete callback will be called with the final status of the get.

              **Arguments:**

              - *url* The url to download.

              - *attr* An OperationAttr object containing the attributes for
              this get.

              - *marker* A RestartMarker object.

              **Returns**
              Nothing is returned
          
              **Raises:**

              A FtpClientException is thrown if unable to initiate the get.
              """
       
        if attr == None:
            attr = OperationAttr()

        if marker == None:
            marker = RestartMarker()

        self._cb_handle = None
        self._buffer = Buffer(64*1024)
        fd = open(dest_file, "w")

        self._condV.acquire()
        ret, cbHandle = ftpClientc.get(self._handle, url, attr.get_handle(),
                                      marker.get_handle(), self.done_cb, self._condV)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        
        ret,handle = ftpClientc.register_read(self._handle,self._buffer.get_handle(),
                                              self._buffer.get_size(), self.read_cb, fd)

        if not ret == 0:
            ex = FtpClientException(handle)
            raise ex
 
        self._condV.wait()
        self._condV.release()
        ftpClientc.free_callback(cbHandle)
        ftpClientc.free_callback(handle)

    def partial_get(self,dest_file, url,offset, endOffset, attr = None, marker = None):
        """
              Get part of a file from a ftp server.
              
              After calling this method, register_read can be called to begin retrieving
              the data.  When all of the data associated with this URL is retrieved, and all
              of the data callbacks have completed, or if the get request is aborted, the
              complete callback will be called with the final status of the get.

              **Arguments:**

              - *url* The url to download.

              - *attr* An OperationAttr object containing the attributes for
              this get.

              - *marker* A RestartMarker object.

              - *offset* An int or long offset to begin the transfer at.

              - *endOffset* An int or long offset to end the transfer at.

              **Returns**
              Nothing is returned
            
              **Raises:**

              A FtpClientException is thrown if unable to initiate the partial get.
              """
        
        if attr == None:
            attr = OperationAttr()

        if marker == None:
            marker = RestartMarker()

        self._cb_handle = None
        self._buffer = Buffer(64*1024)
        fd = open(dest_file, "w")

        self._condV.acquire()
        ret, cbHandle = ftpClientc.partial_get(self._handle, url, attr.get_handle(),
                                              marker.get_handle(), offset,
                                              endOffset, self.done_cb, self._condV)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
        
        ret,handle = ftpClientc.register_read(self._handle,self._buffer,len(self._buffer),
                                              self.read_cb, fd)

        if not ret == 0:
            ex = FtpClientException(handle)
            raise ex
        
        
        self._condV.wait()
        self._condV.release()
        ftpClientc.free_callback(cbHandle)
        ftpClientc.free_callback(handle)
        
    def put(self, src_file,url, attr = None, marker = None):
        """
              Store a file on an ftp server.

              After calling this method, register_write can be called to begin writing
              the data.  When all of the data associated with this URL is sent, and all
              of the data callbacks have completed, or if the put request is aborted, the
              complete callback will be called with the final status of the put.

              **Arguments:**

              - *url* The url to store the data to.

              - *attr* An OperationAttr object containing the attributes for
              this put.
            
              - *marker* A RestartMarker object.

              **Returns**
              Nothing is returned  

              **Raises:**

              A FtpClientException is thrown if unable to initiate the put.
              """

        if attr == None:               
	    attr = OperationAttr()

        if marker == None:
            marker = RestartMarker()

        self._cb_handle = None
        fd = open(src_file, "r")
        
        self._condV.acquire()
        ret, cbHandle = ftpClientc.put(self._handle, url, attr.get_handle(),
                                      marker.get_handle(),self.done_cb, self._condV )

        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex

        size = 64*1024
        oset = 0
        self._buffer = fd.read(size)
       
        ret, handle = ftpClientc.register_write(self._handle, self._buffer, len(self._buffer),
                                           oset, buffer == "",self.write_cb, fd)
        if not ret == 0:
            ex = FtpClientException(handle)
            raise ex

        self._condV.wait()
        self._condV.release()
        ftpClientc.free_callback(cbHandle)
        ftpClientc.free_callback(handle)

    def partial_put(self, src_file, url, offset, endOffset, attr = None, marker= None):
        """
              Put part of a file onto a ftp server.
              
              After calling this method, register_write can be called to begin writing
              the data.  When all of the data associated with this URL is sent, and all
              of the data callbacks have completed, or if the partial_put request is aborted, the
              complete callback will be called with the final status of the partial_put.
              
              **Arguments:**
              
              - *url* The url to download.

              - *attr* An OperationAttr object containing the attributes for
              this partial_put.

              - *marker* A RestartMarker object.

              - *offset* An int representing the byte offset to begin the transfer at.
              
              - *endOffset* An int representing the byte offset to end the transfer at.

              **Returns**
              Nothing is returned
              **Raises:**

              A FtpClientException is thrown if unable to initiate the partial put.
              """
        
        if attr == None:               
	    attr = OperationAttr()

        if marker == None:
            marker = RestartMarker()

        self._cb_handle = None
        size = 64 *1024
        self._buffer = Buffer(size)
        fd = open(src_file, "r")

        self._condV.acquire()
        ret, cbHandle = ftpClientc.partial_put(self._handle, url, attr.get_handle(),
                                              marker.get_handle(), offset,
                                              endOffset, self.done_cb, self._condV)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex

        fd.seek(offset)
        
        if endOffset - offset < size:
            self._buffer = fd.read(endOffset - offset)
        else:
            self._buffer = fd.read(size)
            
            
        ret, handle = ftpClientc.register_write(self._handle, self._buffer, len(self._buffer),
                                           offset, buffer == "",self.write_cb_partial, (fd, endOffset))

        if not ret == 0:
            ex = FtpClientException(handle)
            raise ex
        
        self._condV.wait()
        self._condV.release()
        ftpClientc.free_callback(cbHandle)
        ftpClientc.free_callback(handle)

    def third_party_transfer(self, srcUrl, destUrl,
                             srcAttr = None, destAttr = None, marker = None):
 
        """
              Transfer a file between two ftp servers.
              
              When the transfer is complete, the complete callback will be called with the
              final status of the transfer.
              
              **Arguments:**

              - *srcUrl* The url to transfer.

              - *srcAttr* An OperationAttr object containing the attributes
              for the source.

              - *destUrl* The url to store the data to.

              - *destAttr* An OperationAttr object containing the attributes
              for the destination.
            
              - *marker* A RestartMarker object
            
              **Returns**
              Nothing is returned

              **Raises:**

              A FtpClientException is thrown if unable to initiate the transfer.
              """
        
        if srcAttr == None:
            srcAttr = OperationAttr()

        if destAttr == None:
            destAttr = OperationAttr()

        if marker == None:
            marker = RestartMarker()

        self._condV.acquire()    
        ret, cbHandle = ftpClientc.third_party_transfer(self._handle,
                                                       srcUrl,
                                                       srcAttr.get_handle(),
                                                       destUrl,
                                                       destAttr.get_handle(),
                                                       marker.get_handle(),
                                                       self.done_cb, self._condV)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex

        self._condV.wait()
        self._condV.release()
        ftpClientc.free_callback(cbHandle)

    def partial_third_party_transfer(self, srcUrl,destUrl, offset, endOffset,
                                     srcAttr = None, destAttr = None,marker = None):
        """
              Transfer part of a file between two ftp servers.

              When the transfer is complete, the complete callback will be called with the
              final status of the transfer.

              **Arguments:**

              - *srcUrl* The url to transfer.

              - *srcAttr* An OperationAttr object containing the attributes
              for the source.

              - *destUrl* The url to store the data to.

              - *destAttr* An OperationAttr object containing the attributes
              for the destination.

              - *marker* A RestartMarker object.
              
              - *offset* An int or long offset to start the transfer from.

              - *endOffset* An int or long offset to end the transfer at.

              **Returns**
              Nothing is returned

              **Raises:**

              A FtpClientException is thrown if unable to initiate the transfer.
              """
        
        if srcAttr == None:
            srcAttr = OperationAttr()

        if destAttr == None:
            destAttr = OperationAttr()

        if marker == None:
            marker = RestartMarker()
            
        self._condV.acquire()    
        ret, cbHandle = ftpClientc.partial_third_party_transfer(self._handle,
                                                               srcUrl,
                                                               srcAttr.get_handle(),
                                                               destUrl,
                                                               destAttr.get_handle(),
                                                               marker.get_handle(),
                                                               offset, endOffset,
                                                               self.done_cb, self._condV)
        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex
    
        self._condV.wait()
        self._condV.release()
        ftpClientc.free_callback(cbHandle)

    def size(self, url, attr =  None):
        """
              Retrieves the size of a file on a ftp server

              When the size is retrieved the complete callback is called
              
              **Arguments**
              
              - * url, the location of the file

              - * attr, An OperationAttr object, if none is given then
              the default OperationAttr is created and used

              **Returns**
              the size of the file

              **Raises**

              An FtpClientException if the size cannont be retrieved

              """

           
        if attr == None:
            attr = OperationAttr()

        self._condV.acquire()
        ret, size, cbHandle = ftpClientc.size(self._handle, url, attr.get_handle(),
                                        self.done_cb, self._condV)

        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex

        
        self._condV.wait()
        self._condV.release()
        new_size = Off_t(size)
        ftpClientc.free_callback(cbHandle)
        return new_size.getLong()
    
    def exists(self, url, attr = None):
        """
              Determines whether a file exists on a ftp server

              When the existence has been determined  the complete callback is called
              
              **Arguments**

              - * url, the location of the file
              
              - * attr, An OperationAttr object, if none is given then
              the default OperationAttr is created and used

              **Returns**
              a boolean indicating whether the file exists
              
              **Raises**
              An FtpClientException if the existence cannot be determined
              
              """
        
        if attr == None:
            attr = OperationAttr()

            
        arglist = [self._condV, []]
        self._condV.acquire()
        ret, cbHandle = ftpClientc.exists(self._handle, url, attr.get_handle(),
                                          self.done_cb_exists, arglist)

        if not ret == 0:
            ex = FtpClientException(cbHandle)
            raise ex

        self._condV.wait()
        self._condV.release()
        ftpClientc.free_callback(cbHandle)

        if len(arglist[1]) != 0:
            return 0
        else:
            return 1

    def modification_time(self, url, attr = None):
        """
              Retrieves the size of a file on a ftp server
              
              When the modification time is retrieved the complete callback is called
              
              **Arguments**

              - * url, the location of the file
              
              - * attr, An OperationAttr object, if none is given then
              the default OperationAttr is created and used
              
              **Returns**
              the string of the modification time

              **Raises**
              An FtpClientException if the modification time cannot be retrieved

              """

        if attr == None:
            attr = OperationAttr()

        self._condV.acquire()
        ret, cbHandle, mtime = ftpClientc.modification_time(self._handle, url, attr.get_handle(),
                                                            self.done_cb, self._condV)
        if not ret == 0:
             ex = FtpClientException(cbHandle)
             raise ex

        
        self._condV.wait()
        self._condV.release()
        ftpClientc.free_callback(cbHandle)
        t = Abstime(mtime)
        return t
        

           
class HandleAttr:
    """
    A class to wrap the ftp client handle attr functions.

    It provides get and set methods for all of the various handle
    specific attributes, and hides the underlying memory management.
    """

    def __init__(self, handle = None, own=1):
        """
              Constructs an instance.
              
              Activates the globus ftp client module and creates the ftp handle attr.

              **Arguments**

              - *handle* A SWIG'ized pointer to a globus ftp client handleattr
              pointer. Used to construct an instance from an existing pointer.
              
              - *own* True if the class owns the underlying pointer.
            
              **Raises:**

              A HandleAttr exception is thrown if unable to activate
              the module or create the handle.
              """
        
        self._activatd = 0
        self._own = own
        ret = utilc.globus_module_activate(ftpClientc.get_module())
        if not ret == 0:
            ex = HandleAttrException("Unable to initialize the module")
            raise ex
        self._activated = 1
        if handle == None:
            ret, handle = ftpClientc.handleattr_init()
            if not ret == 0:
                ex = HandleAttrException(handle)
                raise ex
        self._handle = handle

    def __del__(self):
        """
              Destroy an instance.

              Deletes the ftp attr handle and deactivates the underlying globus
              module.

              **Raises:**

              A HandleAttrException is thrown if unable to destroy the
              handle or deactivate the module.
              """
        
        if self._own:
            ret, error = ftpClientc.handleattr_destroy(self._handle)
            if not ret == 0:
                ex = HandleAttrException(error)
                raise ex
        ret = utilc.globus_module_deactivate(ftpClientc.get_module())
        if not ret == 0:
            ex = HandleAttrException("Unable to deactivate the module")
            raise ex

    def copy(self):
        """
              Creates a deep copy of itself.
              
              **Raises:**

              A HandleAttrException is thrown if unable to make the copy.

              **Returns:**

              A new HandleAttr object with the same properties as the
              original is returned.
              """
        
        ret, handle = ftpClientc.handleattr_copy(self._handle)
        if not ret == 0:
            ex = HandleAttrException(handle)
            raise ex
        return HandleAttr(handle)

    def add_cached_url(self, url):
        """
              Enable caching for the specified url.
              
              The handle will keep the connection to the specified url open
              in between ftp operations.

              **Arguments:**

              - *url* The url to cache.

              **Raises:**

              A HandleAttrException is thrown if unable to cache the
              url.
              """
        
        ret, error = ftpClientc.handleattr_add_cached_url(self._handle, url)
        if not ret == 0:
            ex = HandleAttrException(error)
            raise ex
        return

    def remove_cached_url(self, url):
        """
              Disable caching for the specified url.
              
              The handle will no longer keep the connection open to the specified
              url between connections.

              **Arguments:**
              
              - *url* The url to remove from the cache.

              **Raises:**

              A HandleAttrException is thrown if unable to remove the url
              from the cache.
              """
        
        ret, error = ftpClientc.handleattr_remove_cached_url(self._handle, url)
        if not ret == 0:
            ex = HandleAttrException(error)
            raise ex
        return

    def set_cache_all(self, cacheAll):
        """
              Set the cache all attribute.

              If true, all connections on the handle will be cached in between
              connections. When this is enabled, the user skips the authentication
              handshake and connection establishment overhead for multiple subsequent
              ftp operations to the same server.

              **Arguments:**

              - *cacheAll* A boolean describing whether to cache all connections
              or not.

              **Raises:**

              A HandleAttrException is thrown if unable to set the
              attribute.
              """
        
        ret, error = ftpClientc.handleattr_set_cache_all(self._handle,
                                                         cacheAll)
        if not ret == 0:
            ex = HandleAttrException(error)
            raise ex
        return

    def get_cache_all(self):
        """
              Return the cache all attribute.
              
              If true, all connections on the handle will be cached in between
              connections. When this is enabled, the user skips the authentication
              handshake and connection establishment overhead for multiple subsequent
              ftp operations to the same server.
              
              **Raises:**

              A HandleAttrException is thrown if unable to return the
              attribute.
              """
        
        ret, error, cacheAll = ftpClientc.handleattr_get_cache_all(self._handle)
        if not ret == 0:
            ex = HandleAttrException(error)
            raise ex
        return cacheAll
    
    def set_netlogger(self, nl_handle):
        """
              Set netlogger handle.

              **Arguments**

              *nl_handle* A netlogger handle.
              
              **Raises**

              An HandleAttrException is thrown if unable to add the plugin.
              """
        
        ret, error = ftpClientc.handleattr_set_cache_all(self._handle,
                                                         nl_handle)
        if not ret == 0:
            ex = HandleAttrException(error)
            raise ex
        return
    
    def add_plugin(self, plugin):
        """
              Add a plugin to a handle attribute set.
              
              Each handle attribute set contains a list of plugins associated with it. When a
              handle is created with a particular attribute set, it will be associated with a copy
              of those plugins.
              
              Only one instance of a specific plugin may be added to an attribute set. Each
              plugin must have a different name.

              A copy of the plugin is created via the plugins 'copy' method when it is added to
              an attribute set. Thus, any changes to a particular plugin must be done before the
              plugin is added to an attribute set, and before the attribute set is used to create handles.
              
              **Arguments**

              *plugin* A Plugin class.
              
              **Raises**

              An HandleAttrException is thrown if unable to add the plugin.
              """
        
        ret, err = ftpClientc.handleattr_add_plugin(self._handle, plugin._handle)
        if not ret == 0:
            ex = HandleAttrException(err)
            raise ex
        return

    def remove_plugin(self, plugin):
        """
              Remove a plugin from a handle attribute set.
              
              Each handle attribute set contains a list of plugins associated with it. When a
              handle is created with a particular attribute set, it will be associated with a copy
              of those plugins.

              Only one instance of a specific plugin may be added to an attribute set. Each
              plugin must have a different name.

              A copy of the plugin is created via the plugins 'copy' method when it is added to
              an attribute set. Thus, any changes to a particular plugin must be done before the
              plugin is added to an attribute set, and before the attribute set is used to create handles.
              
              **Arguments**

              *plugin* A Plugin class.
              
              **Raises**

              An HandleAttrException is thrown if unable to remove the plugin.
              """
        
        ret, err = ftpClientc.handleattr_remove_plugin(self._handle, plugin._handle)
        if not ret == 0:
            ex = HandleAttrException(err)
            raise ex
        return
    
    def get_handle(self):
        """
              Return the underlying ftp client handle attr handle.
              
              **XXX** This handle is only valid as long as this object exists.  When
              the object is destroyed, the memory this handle points to is freed.
              
              **Returns:**

              A string containing the pointer.
              """
        
        return self._handle
    
class OperationAttr:
    """
    A class to wrap the ftp client operation attr functions.

    It provides get and set methods for all of the various operation
    specific attributes, and hides the underlying memory management.
    """
    
    def __init__(self, handle = None):
        """
              Constructs an instance.
              
              Activates the globus ftp client module and creates the underlying handle.

              **Raises:**

              A OperationAttrException is thrown if unable to initialize the
              module or create the handle.
              """
        
        self._activated = 0
        ret = utilc.globus_module_activate(ftpClientc.get_module())
        if not ret == 0:
            ex = OperationAttrException("Unable to initialize the module")
            raise ex
        self._activated = 1
        if handle == None:
            ret, handle = ftpClientc.operationattr_init()
            if not ret == 0:
                ex = OperationAttrException(handle)
                raise ex
        self._handle = handle

    def __del__(self):
        """
              Destroy an instance.

              Deletes the underlying handle, and deactivates the underlying module.

              **Raises:**

              A OperationAttrException is thrown if unable to delete the
              handle or destroy the module.
              """
       
	if ftpClientc == None:
	     return 
     
	ret, error = ftpClientc.operationattr_destroy(self._handle)
        if not ret == 0:
            ex = OperationAttrException(error)
            raise ex
        ret = utilc.globus_module_deactivate(ftpClientc.get_module())
        if not ret == 0:
            ex = OperationAttrException("Unable to deactivate the module")
            raise ex


    def copy(self):
        """
              Creates a deep copy of itself.
              
              **Raises:**

              A OperationAttrException is thrown if unable to make the copy.

              **Returns:**
              
              A new OperationAttr object with the same properties as the
              original is returned.
              """
        
        ret, handle = ftpClientc.operationattr_copy(self._handle)
        if not ret == 0:
            ex = OperationAttrException(handle)
            raise ex
        return OperationAttr(handle)

    def set_append(self, append):
        """
              Set the append mode for the operation.

              Allows a file to be appended to when performing a put to a server instead of
              replacing the file. This attribute is incompatible with the extended block mode,
              the partial file attribute, and the restart attribute.
              
              **Arguments:**

              - *append* A boolean value.

              **Raises:**

              A OperationAttrException is thrown if unable to set the append
              attribute.
              """
        
        ret, error = ftpClientc.operationattr_set_append(self._handle, append)
        if not ret == 0:
            ex = OperationAttrException(error)
            raise ex
        return

    def get_append(self):
        """
              Returns the value of the append attribute.

              **Raises:**

              A OperationAttrException is thrown if unable to return the append
              attribute.
            
              **Returns:**

              A boolean value is returned.
              """
        
        ret, val = ftpClientc.operationattr_get_append(self._handle)
        if not ret == 0:
            ex = OperationAttrException(val)
            raise ex
        return val

    def set_authorization(self, credential, user=None, passwd="", account="", subject=""):
        """
              Set the authorization attributes.
              
              **Arguments:**
              
              - *user* The user name to send to the server.

              - *credential* A SWIG'ized pointer to a gss_cred_id_t object. To use the
              default credential, use GSS_C_NO_CREDENTIAL. This can be returned
              from the gsi module.

              - *passwd* A string containing the password to send to the server.
              
              - *account* The account to use for the data transfer. Most ftp servers do not
              require this.

              - *subject* The subject name of the ftp server. This is only needed for
              a gsiftp transfer when the security subject name does not match the subject
              name of the server.

              **Raises:**

              A OperationAttrException is thrown if uanbe to set the
              authorization attributes.
              """
        
        ret, error = ftpClientc.operationattr_set_authorization(self._handle, credential._handle,
                                                                user, passwd, account, subject)
        if not ret == 0:
            print "ret is %i" % ret
            print "error is %s" % error
            ex = OperationAttrException(error)
            raise ex
        return

    def get_authorization(self):
        """
              Returns the authorization attributes.
              
              **Raises:**

              A OperationAttrException is thrown if unable to get the authorization
              attributes.

              **Returns:**
              
              - A SWIG'ized pointer to a gss_ctx_id_t object.

              - A string containg the user name.
              
              - A string containing the password.

              - A string containing the account name.
              
              - A string containing the subject name of the server.
              """
        
        ret, credential, user, passwd, account, subject = ftpClientc.operationattr_get_authorization(self._handle)
        if not ret[0] == 0:
            ex = OperationAttrException(ret[1])
            raise ex
        return credential, user, passwd, account, subject

    def set_dcau(self, dcau):
        ret, err = ftpClientc.operationattr_set_dcau(self._handle, dcau.get_handle())
        if not ret == 0:
            ex = OperationAttrException(err)
            raise ex
        return

    def get_dcau(self):
        ret, dcau = ftpClientc.operationattr_get_dcau(self._handle)
        if not ret == 0:
            ex = OperationAttrException(dcau)
            raise ex
        return FtpControlDCAU.FtpControlDCAU(dcau)

    def set_data_protection(self, protection):
        """
              Sets the protection attribute for the data channel.
              
              The supported protection attributes are defined in the ftpControl module.
              
              **Arguments:**

              - *protection* One of the possible attributes defined in the ftpControl
              module.

              **Raises:**

              A OperationAttrException is thrown if unable to set the data protection
              attribute.
              """
        
        ret, error = ftpClientc.operationattr_set_data_protection(self._handle, protection)
        if not ret == 0:
            ex = OperationAttrException(error)
            raise ex
        return

    def get_data_protection(self):
        """
              Returns the protection attribute for the data channel.
              
              **Raises:**
              
              A OperationAttrException is thrown if unable to get the attribute.

              **Returns:**

              One of the attributes defined in the ftpControl module is returned.
              """
        
        ret, protection = ftpClientc.operationattr_get_data_protection(self._handle)
        if not ret == 0:
            ex = OperationAttrException(protection)
            raise ex
        return mode

    def set_control_protection(self, protection):
        """
              Sets the protection attribute for the control channel.

              The supported protection attributes are defined in the ftpControl module.
              
              **Arguments:**

              - *protection* One of the possible attributes defined in the ftpControl
              module.

              **Raises:**
              
              A OperationAttrException is thrown if unable to set the control protection
              attribute.
              """
        
        ret, error = ftpClientc.operationattr_set_control_protection(self._handle, protection)
        if not ret == 0:
            ex = OperationAttrException(error)
            raise ex
        return

    def get_control_protection(self):
        """
              Returns the protection attribute for the control channel.

              **Raises:**
              
              A OperationAttrException is thrown if unable to get the attribute.

              **Returns:**

              One of the attributes defined in the ftpControl module is returned.
              """
        
        ret, protection = ftpClientc.operationattr_get_control_protection(self._handle)
        if not ret == 0:
            ex = OperationAttrException(protection)
            raise ex
        return mode
    
    def set_layout(self, layout):
        """
              Sets the layout attribute for the transfer.

              This attribute allows the user to control the layout of a file being transfered to a
              striped Grid-FTP server. The striping layout defines what regions of a file will be
              stored on each stripe of a multiple-striped ftp server. 
              
              The layout attribute is used only when the data is being stored on the server
              (on a put or 3rd party transfer). This attribute is ignored for stream mode data
              transfers.
              
              **Arguments:**
              
              - *layout* A ftpControl.Layout object.

              **Raises:**
              
              A OperationAttrException is thrown if unable to set the layout
              attribute.
              """
        
        ret, error = ftpClientc.operationattr_set_layout(self._handle,
                                               layout.get_handle())
        if not ret == 0:
            ex = OperationAttrException(error)
            raise ex
        return

    def get_layout(self):
        """
              Returns the layout attribute for this operation.

              **Raises:**
              
              A OperationAttrException is thrown if unable to get the layout
              attribute.

              **Returns:**

              A ftpControl.Layout object is returned.
              """
        
        ret, layHandle = ftpClientc.operationattr_get_layout(self._handle)
        if not ret == 0:
            ex = OperationAttrException(layHandle)
            raise ex
        return Layout(layHandle)

    def set_mode(self, mode):
        """
              Sets the mode for the transfer.

              The supported modes are defined in the ftpControl module.
              
              **Arguments:**

              - *mode* One of the possible modes defined in the ftpControl
              module.
              
              **Raises:**

              A OperationAttrException is thrown if unable to set the mode.
              """
        
        ret, error = ftpClientc.operationattr_set_mode(self._handle, mode)
        if not ret == 0:
            ex = OperationAttrException(error)
            raise ex
        return

    def get_mode(self):
        """
              Returns the mode for this transfer.

              **Raises:**
              
              A OperationAttrException is thrown if unable to get the mode.

              **Returns:**

              One of the modes defined in the ftpControl module is returned.
              """
        
        ret, mode = ftpClientc.operationattr_get_mode(self._handle)
        if not ret == 0:
            ex = OperationAttrException(mode)
            raise ex
        return mode

    def set_parallelism(self, parallelism):
        """
              Set the parallelism attribute.

              Used to control the level of parallelism used for the operation.

              **Arguments:**

              - *parallelism* A ftpContol.Parallelism object.

              **Raises:**

              A OperationAttrException is thrown if unable to set the
              parallelism attribute.
              """
        
        ret, error = ftpClientc.operationattr_set_parallelism(self._handle,
                                                    parallelism.get_handle())
        if not ret == 0:
            ex = OperationAttrException(errpr)
            raise ex
        return

    def get_parallelism(self):
        """
              Returns the parallelism attribute.

              **Raises:**

              A OperationAttrException is thrown if unable to get the
              parallelism attribute.
              
              **Returns:**
              
              A ftpContol.Parallelism object is returned.
              """
        
        ret, parHandle = ftpClientc.operationattr_get_parallelism(self._handle)
        if not ret == 0:
            ex = OperationAttrException(parHandle)
            raise ex
        return Parallelism(parHandle)

    def set_tcp_buffer(self, tcpbuffer):
        """
              Set the tcp buffer attribute.
              
              Used to control the tcp buffer size for the operation.
              
              **Arguments:**
        
              - *tcpbuffer* A ftpContol.TcpBuffer object with the proper 
              buffer size set.
              
              **Raises:**

              A OperationAttrException is thrown if unable to set the tcp buffer
              attribute.
              """
        
        ret, error = ftpClientc.operationattr_set_tcp_buffer(self._handle,
                                                   tcpbuffer.get_handle())
        if not ret == 0:
            ex = OperationAttrException(error)
            raise ex
        return

    def get_tcp_buffer(self):
        """
              Return the tcp buffer attribute.
              
              **Raises:**
              
              A OperationAttrException is thrown if unable to get the tcp buffer
              attribute.
            
              **Returns:**

              A ftpContol.TcpBuffer object is returned.
              """
        
        ret, bufHandle = ftpClientc.operationattr_get_tcp_buffer(self._handle)
        if not ret == 0:
            ex = OperationAttrException(bufHandle)
            raise ex
        return TcpBuffer(bufHandle)

    def set_type(self, type):
        """
              Sets the type attribute for the operation.

              Controls the type used for the transfer. The types are defined in the 
              ftpControl module. The default type is binary.

              **Arguments:**

              - *type* One of the types defined in the ftpControl module.

              **Raises:**

              A OperationAttrException is thrown if unable to set the type.
              """
        
        ret, error = ftpClientc.operationattr_set_type(self._handle, type)
        if not ret == 0:
            ex = OperationAttrException(error)
            raise ex
        return

    def get_type(self):
        """
              Returns the type attribute for the operation.
              
              **Raises:**
              
              A OperationAttrException is thrown if unable to get the type
              attribute.
              
              **Returns:**
        
              One of the types defined in the ftpControl module is returned.
              """
        
        ret, type = ftpClientc.operationattr_get_type(self._handle)
        if not ret == 0:
            ex = OperationAttrException(type)
            raise ex
        return type

    def get_handle(self):
        """
              Return the underlying ftp client operation attr handle.

              **XXX** This handle is only valid as long as this object exists.  When
              the object is destroyed, the memory this handle points to is freed.
              
              **Returns:**

              A string containing the pointer.
              """
        
        return self._handle



    
class RestartMarker:
    """
    A class to wrap the ftp client restart marker attributes.

    It provides various get and set methods to control the properties of a transfer
    that is about to restart.
    """
    
    def __init__(self, handle = None):
        """
              Constructs an instance.

              Activates the globus ftp client module, and creates the underlying handle.
              
              **Arguments:**

              - *handle* An optional argument used to construct an instance from an
              existing handle. The class assumes it then \"owns\" the handle, and
              can free it when it is destroyed.

              **Raises:**

              A RestartMarkerException is thrown if unable to initialize the
              module, or create the handle.
              """
              
        self._activated = 0
        ret = utilc.globus_module_activate(ftpClientc.get_module())
        if not ret == 0:
            ex = RestartMarkerException("Unable to initialize the module")
            raise ex
        self._activated = 1
        if handle == None:
            ret, handle = ftpClientc.restart_marker_init()
            if not ret == 0:
                ex = RestartMarkerException(handle)
                raise ex
        self._handle = handle
        return

    def __del__(self):
        """
              Destroy an instance.

              Destroys the underlying handle, and deactivates the module.

              
              NOTE: This destructor is sometimes called when the ftpClientc has already
              been destroyed because of somewhat unpleasant python behavior when
              this module is imported

              **Raises:**

              A RestartMarkerException is thrown if unable to deactivate
              the module 
              """
        
        if ftpClientc == None:
            return

	ftpClientc.restart_marker_destroy(self._handle)
        ret = utilc.globus_module_deactivate(ftpClientc.get_module())
        if not ret == 0:
            ex = RestartMarkerException("Unable to deactivate the module")
            raise ex
        return

    def copy(self):
        """
              Creates a deep copy of iteself.

              **Raises:**
              
              A RestartMarker is thrown if unable to make the copy.

              **Returns:**

              A new RestartMarker object with the same attributes as
              the old is returned.
              """
        
        ret, handle = ftpClientc.restart_marker_copy(self._handle)
        if not ret == 0:
            ex = RestartMarkerException(handle)
            raise ex
        return RestartMarker(handle)

    def insert_range(self, offset, endOffset):
        """
              Insert a range into the marker.
              
              This method updates the marker with a new byte range, suitable for using
              to restart an extended block mode transfer. Adjacent ranges within the marker
              will be combined into a single entry in the marker.
              
              A marker can only hold a range list or a stream offset. Calling this method after
              calling set_offset() will result in a marker suitable only for use restarting an
              extended block mode transfer.
              
              **Arguments:**

              - *offset* The beginning offset to start at. This may be an int or a python
              long, if your platform supports 64 bit files.
              
              - *endOffset* The offset to end at.

              **Raises:**
              
              A RestartMarkerException is thrown if unable to insert the
              restart range.
              """
        
        ret, error = ftpClientc.restart_marker_insert_range(self._handle,
                                                           offset,
                                                           endOffset)
        if not ret == 0:
            ex = RestartMarkerException(error)
            raise ex
        return

    def set_offset(self, offset):
        """
              Insert the offset to restart at.
              
              This method modifies the marker to contain a stream offset, suitable for
              using to restart a steam mode transfer.
              
              A marker can only hold a range list or a stream offset. Calling this method after
              calling insert_range() will delete the ranges associated with the marker, and
              replace it with a marker suitable only for use restarting a stream mode transfer. 
              
              When restarting an ASCII type transfer, the offset must take into account the
              additional carriage return characters added to the data stream.
              
              **Arguments:**

              - *offset* The offset to begin at. This may be an int, or a python long
              on platformst that support 64 bit files.
              
              **Raises:**
              
              A RestartMarkerException is thrown if unable to set the
              offset.
              """
        
        ret, error = ftpClientc.restart_marker_set_offset(self._handle,
                                                         offset)
        if not ret == 0:
            ex = RestartMarkerException(error)
            raise ex
        return

    def set_ascii_offset(self, offset, asciiOffset):
        """
              Set the offset for a restart marker.
              
              This method modifies the marker to contain a stream offset, suitable for
              using to restart a steam mode transfer.
        
              A marker can only hold a range list or a stream offset. Calling this method after
              calling insert_range() will delete the ranges associated with the marker, and
              replace it with a marker suitable only for use restarting a stream mode transfer. 
              
              When restarting an ASCII type transfer, use this method to set both the offset
              used in the local representation of an ACSII file, and the network representation
              of the ASCII file. For UNIX systems, the former includes counts newlines as one
              character towards the file offset, and the latter counts them as 2 characters
              (CRLF).
              
              **Arguments:**

              - *offset* The local stream offset.

              - *asciiOffset:** The networks ASCII representation of the offset.
              
              **Raises:**

              A RestartMarkerException is thrown if unable to set the
              ASCII offset.
              """
        
        ret, error = ftpClientc.restart_marker_set_ascii_offset(self._handle,
                                                               offset,
                                                               asciiOffset)
        if not ret == 0:
            ex = RestartMarkerException(error)
            raise ex
        return

    def get_total(self):
        """
              Get total bytes accounted for in restart marker.
              
              This funtion will return the sum of all bytes accounted for in a restart
              marker. If this restart marker contains a stream offset then this value
              is the same as the offset (not the ascii offset) that it was set with. If it
              is a range list, it a sum of all the bytes in the ranges.
        
              **Raises:**

              A RestartMarkerException is thrown if unable to get the
              total bytes.

              *Returns:**

              The total bytes in the marker..
              """
        
        ret, total = ftpClientc.restart_marker_get_total(self._handle)
        if not ret == 0:
            ex = RestartMarkerException(total)
            raise ex
        return total
    
    def to_string(self):
        """
              Return a string represenation of the marker.

              Returns a string suitable for sending to the FTP REST command, or for a
              later call to from_string().
              
              **Raises:**

              A RestartMarkerException is thrown if unable to get the
              marker string.

              *Returns:**
        
              A string representation of the marker.
              """
        
        ret, error, str = ftpClientc.restart_marker_to_string(self._handle)
        if not ret == 0:
            ex = RestartMarkerException(error)
            raise ex
        return str

    def from_string(self, markerStr):
        """
              Initialize the marker from a string.

              **Raises:**
              
              A RestartMarkerException is thrown if unable to initialize
              the marker from the given string.
              """
        
        ret, error = ftpClientc.restart_marker_from_string(self._handle,
                                                          markerStr)
        if not ret == 0:
            ex = RestartMarkerException(error)
            raise ex
        return

    def get_handle(self):
        """
              Return the underlying ftp client restart marker handle.

              **XXX** This handle is only valid as long as this object exists.  When
              the object is destroyed, the memory this handle points to is freed.
              
              **Returns:**
        
              A string containing the pointer.
              """
        
        return self._handle

class Plugin:
    """
       A base class for the ftp client plugins.
       """
    
    def __init__(self):
        self._handle = None
        return
    
    def get_handle(self):
        """
        Return the underlying ftp client plugin handle.

        **XXX** This handle is only valid as long as this object exists.  When
        the object is destroyed, the memory this handle points to is freed.

        **Returns:**

            A string containing the pointer.
        """
        return self._handle

class DebugPlugin(Plugin):
    """
     A wrapper class around the ftp client debug plugin.

     The FTP Debugging plugin provides a way for the user to trace FTP
     protocol messages which occur while the GridFTP client library processes
     an FTP operation. This may be useful for debugging FTP configuration problems.

     When this plugin is used for a GridFTP Client operation, information will be printed t
     o the file stream associated with the plugin when a user begins an operation, for all
     data buffers which pass through while handling a data transfer, and for all protocol
     messages which are sent and received.
     """

    def __init__(self, file, text):
        """
        Initialize an instance of the GridFTP debugging plugin.

        This function will initialize the debugging plugin-specific instance data for this
        plugin, and will make the plugin usable for ftp client handle attribute and
        handle creation.

        **Arguments**

            *file* An open Python file object to write the debug messages to.

            *text* A string message to append to the debug info.

        **Raises**

            A PluginException is thrown if unable to init the plugin.
        """
        Plugin.__init__(self)
        ret = utilc.globus_module_activate(ftpClientc.get_debug_plugin_module())
        if not ret == 0:
            ex = PluginException("Unable to activate the module")
            raise ex
        ret, handle = ftpClientc.debug_plugin_init(file, text)
        if not ret == 0:
            ex = PluginException(handle)
            raise ex
        self._handle = handle
        return

    def __del__(self):
        """
        Destroy an instance of the GridFTP debugging plugin.

        This function will free all debugging plugin-specific instance data from this
        plugin, and will make the plugin unusable for further ftp handle creation.

        Existing FTP client handles and handle attributes will not be affected by
        destroying a plugin associated with them, as a local copy of the plugin is
        made upon handle initialization.

        **Raises**

            A PluginException is thrown if unable to destroy the plugin.
        """
        ret, err = ftpClientc.debug_plugin_destroy(self._handle)
        if not ret == 0:
            ex = PluginException(err)
            raise ex
        ret = utilc.globus_module_deactivate(ftpClientc.get_debug_plugin_module())
        if not ret == 0:
            ex = PluginException("unable to deactivate the module")
            raise ex
        return

class RestartMarkerPlugin(Plugin):
    """
    A wrapper class around the ftp client restart marker plugin.
    
    This plugin is intended to allow users to make restart markers persistant.
    During a transfer, every marker received will result in the user's 'marker'
    callback being called with the new restart marker that can be stored. If the
    application were to prematurely terminate (while transferring), the user
    (after restarting the application) could pass this stored marker back to the
    plugin via the 'begin' callback to force the transfer to be restarted from
    the last marked point.
    """

    def __init__(self, beginCB, markerCB, completeCB, arg):
        """
        Initialize an instance of the GridFTP restart marker plugin.

        This function will initialize the resart marker plugin-specific instance data for this
        plugin, and will make the plugin usable for ftp client handle attribute and
        handle creation.

        **Arguments**

            *beginCB* The callback to be called upon the start of the transfer..

            *markerCB* The callback to be called with each restart marker
            received.

            *completeCB* The callback to be called to indicate transfer
            completion.

            *arg* An arbitary argument to be passed to all the callbacks.

        **Raises**

            A PluginException is thrown if unable to init the plugin.
        """
        Plugin.__init__(self)
        self._cbHandle = None
        ret = utilc.globus_module_activate(ftpClientc.get_restart_marker_plugin_module())
        if not ret == 0:
            ex = PluginException("Unable to activate the module")
            raise ex
        ret, handle, cbHandle = ftpClientc.restart_marker_plugin_init(beginCB,
                                                                      markerCB, completeCB, arg)
        if not ret == 0:
            ex = PluginException(handle)
            raise ex
        self._handle = handle
        self._cbHandle = cbHandle
        return

    def __del__(self):
        """
        Destroy an instance of the GridFTP restart marker plugin.

        This function will free all debugging plugin-specific instance data from this
        plugin, and will make the plugin unusable for further ftp handle creation.

        Existing FTP client handles and handle attributes will not be affected by
        destroying a plugin associated with them, as a local copy of the plugin is
        made upon handle initialization.

        **Raises**

            A PluginException is thrown if unable to destroy the plugin.
        """
        ret, err = ftpClientc.restart_marker_plugin_destroy(self._handle)
        if not ret == 0:
            ex = PluginException(err)
            raise ex
        if not self._cbHandle == None:
            ftpClientc.free_callback2(self._cbHandle)
        ret = utilc.globus_module_deactivate(ftpClientc.get_restart_marker_plugin_module())
        if not ret == 0:
            ex = PluginException("unable to deactivate the module")
            raise ex
    

class PerformanceMarkerPlugin(Plugin):
    """
    A wrapper class around the ftp client performance marker plugin.
    
    This plugin provides performance marker information via callbacks on all
    types of transfers except third_party_transfers w/o extended block mode 
    transfer.  During a transfer, every marker received will result in the
    user's 'marker' callback being called with new performance markers that
    can be stored.
    """

    def __init__(self, beginCB, markerCB, completeCB, arg):
        """
        Initialize an instance of the GridFTP restart marker plugin.

        This function will initialize the resart marker plugin-specific instance data for this
        plugin, and will make the plugin usable for ftp client handle attribute and
        handle creation.

        **Arguments**

            *beginCB* The callback to be called upon the start of the transfer..

            *markerCB* The callback to be called with each restart marker
            received.

            *completeCB* The callback to be called to indicate transfer
            completion.

            *arg* An arbitary argument to be passed to all the callbacks.

        **Raises**

            A PluginException is thrown if unable to init the plugin.
        """
        Plugin.__init__(self)
        self._cbHandle = None
        ret = utilc.globus_module_activate(ftpClientc.get_performance_marker_plugin_module())
        if not ret == 0:
            ex = PluginException("Unable to activate the module")
            raise ex
        ret, handle, cbHandle = ftpClientc.performance_marker_plugin_init(beginCB,
                                                                          markerCB, completeCB, arg)
        if not ret == 0:
            ex = PluginException(handle)
            raise ex
        self._handle = handle
        self._cbHandle = cbHandle
        return

    def __del__(self):
        """
        Destroy an instance of the GridFTP restart marker plugin.

        This function will free all debugging plugin-specific instance data from this
        plugin, and will make the plugin unusable for further ftp handle creation.

        Existing FTP client handles and handle attributes will not be affected by
        destroying a plugin associated with them, as a local copy of the plugin is
        made upon handle initialization.

        **Raises**

            A PluginException is thrown if unable to destroy the plugin.
        """
        ret, err = ftpClientc.performance_marker_plugin_destroy(self._handle)
        if not ret == 0:
            ex = PluginException(err)
            raise ex
        if not self._cbHandle == None:
            ftpClientc.free_callback2(self._cbHandle)
        ret = utilc.globus_module_deactivate(ftpClientc.get_performance_marker_plugin_module())
        if not ret == 0:
            ex = PluginException("unable to deactivate the module")
            raise ex

class ThoughputPerfPlugin(Plugin):
    """
    A wrapper class around the ftp client throughput performance plugin.
    This plugin is built on top of the Performance Marker Plugin, it is
    not possible to associate both plugins with a handle 
    
    This plugin provides throughput performance information via callbacks on
    all types of transfers except third_party_transfers w/o extended block
    mode transfer.  During a transfer, every marker received will result in
    the user's 'marker' callback being called with new performance markers
    that can be stored.
    """

    def __init__(self, beginCB, stripeCB, totalCB, completeCB, arg):
        """
        Initialize an instance of the GridFTP Throughput plugin.

        This function will initialize the Throughput plugin-specific instance data
        for this plugin, and will make the plugin usable for ftp client handle
        attribute and handle creation.

        **Arguments**

            *beginCB* The callback to be called upon the start of the transfer..

            *stripeCB* The callback to be called with every performance callback
            that is received by the perf plugin. The first callback for each
            stripe_ndx will have an instantaneous_throughput based from the time
            the command was sent.

            *totalCB* The callback will be called with every performance callback
            that is received by the perf plugin. The first callback for will have
            an instantaneous_throughput based from the time the command was sent.
            This callback will be called after the per_stripe_cb
     
            *completeCB* The callback to be called to indicate transfer
            completion.

            *arg* An arbitary argument to be passed to all the callbacks.

        **Raises**

            A PluginException is thrown if unable to init the plugin.
        """
        Plugin.__init__(self)
        self._cbHandle = None

        ret = utilc.globus_module_activate(ftpClientc.get_throughput_plugin_module())
        if not ret == 0:
            ex = PluginException("Unable to activate the module")
            raise ex
        ret, handle, cbHandle = ftpClientc.throughput_plugin_init(beginCB, stripeCB,
                                                                  totalCB, completeCB, arg)
        if not ret == 0:
            ex = PluginException(handle)
            raise ex
        self._handle = handle
        self._cbHandle = cbHandle
        return

    def __del__(self):
        """
        Destroy an instance of the GridFTP restart marker plugin.

        This function will free all debugging plugin-specific instance data from this
        plugin, and will make the plugin unusable for further ftp handle creation.

        Existing FTP client handles and handle attributes will not be affected by
        destroying a plugin associated with them, as a local copy of the plugin is
        made upon handle initialization.

        **Raises**

            A PluginException is thrown if unable to destroy the plugin.
        """
        ret, err = ftpClientc.throughput_plugin_destroy(self._handle)
        if not ret == 0:
            ex = PluginException(err)
            raise ex
        if not self._cbHandle == None:
            ftpClientc.free_callback3(self._cbHandle)
        ret = utilc.globus_module_deactivate(ftpClientc.get_throughput_plugin_module())
        if not ret == 0:
            ex = PluginException("unable to deactivate the module")
            raise ex
        
class NetloggerPlugin(Plugin):
    """
    A wrapper class around the netlogger plugin.

    """

    def __init__(self, nl_url, prog_name, opaque_string=None):
        """
        Initialize an instance of the netlogger plugin.

        This function will initialize the NetloggerPlugin instance data for this
        plugin, and will make the plugin usable for ftp client handle attribute and
        handle creation.

        **Arguments**
        
          *nl_url*  the url to log to. (May be None)
          Valid urls are:  file://tmp/netlog.log
                           x-netlog://host[:port]
                           x-syslog://localhost
          *prog_name* This is used as the prog name in the NetLoggerOpen call
          
          *opaque_string*  this is an opaque string that will be inserted into
          all logged statements. (may be None)

        **Raises**

            A PluginException is thrown if unable to init the plugin.
        """
        Plugin.__init__(self)
        ret = utilc.globus_module_activate(ftpClientc.get_netlogger_plugin_module())
        if ret != 0:
            raise PluginException("Unable to activate the module")
        ret, handle = \
             ftpClientc.netlogger_plugin_init(nl_url, prog_name, opaque_string)
        if ret != 0:
            raise PluginException(handle)
        self._handle = handle
        return

    def __del__(self):
        """
        Destroy an instance of the netlogger plugin.

        This function will free all debugging plugin-specific instance data from this
        plugin, and will make the plugin unusable for further ftp handle creation.

        Existing FTP client handles and handle attributes will not be affected by
        destroying a plugin associated with them, as a local copy of the plugin is
        made upon handle initialization.

        **Raises**

            A PluginException is thrown if unable to destroy the plugin.
        """
        ret, err = ftpClientc.netlogger_plugin_destroy(self._handle)
        if ret != 0:
            raise PluginException(err)
        ret = utilc.globus_module_deactivate(ftpClientc.get_netlogger_plugin_module())
        if ret != 0:
            raise PluginException("unable to deactivate the module")


