############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
"""
Provides support for the lower-level Ftp Control module.
"""

from pyGlobus import utilc
from pyGlobus.util import GlobusException
import ftpControlc

###############################################################################
# Module constants
##############################################################################
MODE_NONE           = 0
MODE_STREAM         = 'S'
MODE_BLOCK          = 'B'
MODE_EXTENDED_BLOCK = 'E'
MODE_COMPRESSED     = 'C'

TYPE_NONE    = 0
TYPE_ASCII   = 'A'
TYPE_EBCDIC  = 'E'
TYPE_IMAGE   = 'I'
TYPE_LOCAL   = 'L'

STRIPING_NONE = 0
STRIPING_PARTITIONED = 1
STRIPING_BLOCKED_ROUND_ROBIN = 2

PARALLELISM_NONE  = 0
PARALLELISM_FIXED = 1

PROTECTION_CLEAR        = 'C'
PROTECTION_SAFE         = 'S'
PROTECTION_CONFIDENTIAL = 'E'
PROTECTION_PRIVATE      = 'P'

DCAU_NONE = 'N'
DCAU_SELF = 'A'
DCAU_SUBJECT = 'S'

#############################################################################
# Module classes
#############################################################################

class FtpControlException(GlobusException):
    pass

class DCAUException(GlobusException):
    pass

class LayoutException(GlobusException):
    pass

class ParallelismException(GlobusException):
    pass

class TcpBufferException(GlobusException):
    pass

class ResponseException(GlobusException):
    pass

class AuthInfoException(GlobusException):
    pass

class CommandException(GlobusException):
    pass

class FtpControl:

    def __init__(self, handle = None):
        self._activated = 0
	self._cbList = []
        ret = utilc.globus_module_activate(ftpControlc.get_module())
        if not ret == 0:
            ex = FtpControlException("Unable to initialize the module")
            raise ex
        self._activated = 1
        if handle == None:
            ret, handle = ftpControlc.handle_init()
        if not ret == 0:
            ex = FtpControlException("unable to create the handle")
            raise ex
        self._handle = handle
        return

    def __del__(self):
        if not self._handle == None:
            ret = ftpControlc.handle_destroy(self._handle)
            print "del handle: ret is ", ret
            if not ret == 0:
                ex = FtpControlException("unable to destroy the control handle")
                raise ex
        ret = utilc.globus_module_deactivate(ftpControlc.get_module())
        print "del module: ret is ", ret
        if not ret == 0:
            ex = FtpControlException("Unable to deactivate the module")
            raise ex
        for cb in self._cbList:
            self.free_callback(cb)

        return

    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed.
    #######################################################################
    def get_handle(self):
        return self._handle

    ####
    #
    # Jason starts to mess up Keiths code here!
    #
    ####

    def data_connect(self, callback, arg):
        ret, cbHandle = ftpControlc.data_connect_write( self._handle,
                                                       callback, arg)
        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def data_write(self, buffer, length, offset, eof, callback, arg):
        ret, cbHandle = ftpControlc.data_connect_write( self._handle,
                                                       buffer, length,
                                                       offset, eof,
                                                       callback, arg)

        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def local_pasv(self, address):
        ret, err  = ftpControlc.local_pasv( self._handle, address)
        if not ret == 0:
            ex = FtpControlException(err)
            raise ex
        return 
        
    def local_port(self, port):
        ret, err  = ftpControlc.local_port( self._handle, port)
        if not ret == 0:
            ex = FtpControlException(err)
            raise ex
        return 
    
    def read_commands(self, callback, arg):
        ret, cbHandle = ftpControlc.read_commands( self._handle,
                                                  callback, arg)
        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle
    
    def send_response(self, resp, callback, arg):
        ret, cbHandle = ftpControlc.send_response( self._handle, resp, callback, arg)
        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

    def raw_command(self, command):
        ret = ftpControlc.raw_command(command)
        return ret

    def code(self, command):
        ret = ftpControlc.code(command)
        return ret

    def operation(self, command):
        ret = ftpControlc.operation(command)
        return ret

    #TODO should the write info object be in its own class, or a member variable for the ftpControl class?
    def create_data_info(self,buffer,offset, eof, callback, arg):
        """
             Creates a write_info data structure,
             **args**
             *buffer*  is a pyGlobus.util Buffer object
             *offset*  file offset of the data specified
             *eof*     boolean denoting whether an end of file has been reached
             *callback* a python function that has the signature func(user_arg, handle, error, buffer, length, offset, eof)
             *arg*      a user supplied arguement to the callback function

             **Returns**
             returns a SWIG'ized callback handle, and a SWIG'ized write_info pointer

             **Throws**
             throws a FtpControlException if the write_info object could not be created
             """
        ret, cbHandle, write_info = ftpControlc.create_data_info(self._handle, buffer.get_handle(),
                                                     buffer.get_size(), offset, eof, callback, arg)
        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle, write_info

    def release_data_info(self, write_info):
        """Releases the memory associated with the write_info object
                   Must be called after each  create_data_info call

                   **args**
                   *write_info* a SWIG'ized pointer to a write_info object

                   **Throws**
                   throws a FtpControlException if the memory to the write_info object cannot be released

                   **Returns**
                   nothing
                   """


        ret, error = ftpControlc.release_data_info(self._handle, write_info)
        if not ret == 0:
            ex = FtpControlException(error)
            raise ex

        return


    def data_write_stripe(self,buffer,offset,eof,stripe_ndx, callback, arg):
        """
              Writes data to the specified stripe

              **args**
              *buffer* a pyGlobus.util buffer object
              *offset* the offset into file that has been written so far
              *eof* boolean flag if it is the last data to be written
              *stripe_ndx* the stripe which the data is written to.
              *callback* a python function that has the signature func(user_arg, handle, error, buffer, length, offset, eof)
              *arg*      a user supplied arguement to the callback function
              
              **Returns**
              returns a SWIG'ized callback handle

              **Throws**
              throws a FtpControlException if the data could not be written
              """
        ret, cbHandle = ftpControlc.data_write_stripe(self._handle, buffer.get_handle(),buffer.get_size(),offset, eof, stripe_ndx, callback, arg)
        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex

        self._cbList.append(cbHandle)
        return cbHandle


    def x_data_write_stripe(self, buffer, offset, eof, stripe_ndx,write_info):
        """Writes data to the specified stripe
              **args**
              *buffer* a pyGlobus.util buffer object
              *offset* the offset into file that has been written so far
              *eof* boolean flag if it is the last data to be written
              *stripe_ndx* the stripe which the data is written to.
              *write_info* a SWIG'ized pointer to a write_info object

              *Returns*
              nothing
              
              **Throws**
              throws a FtpControlException if the data could not be written
              """

        ret, error = ftpControlc.x_data_write_stripe(self._handle, buffer.get_handle(),buffer.get_size(),offset, eof, stripe_ndx,write_info)
        if not ret == 0:
            ex = FtpControlException(error)
            raise ex

        return

    def control_connect(self, host, port, callback, arg):
        """
              Create a new control connectiong to an FTP server.
              **args**
              *host*      the hostname on a FTP Server
              *port*       the TCP port number on a FTP Server
              *callback* a Python function that has the signature func(user_arg,handle, error, ftp_response)
              *arg*        the user_arg to the callback function

              *Returns*
               returns a SWIG'ized callback handle

               *Throws*
               throws a FtpControlException if a connection could not be initiated
              """
        ret, cbHandle = ftpControlc.control_connect(self._handle,host, port, callback, arg)
        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex

        self._cbList.append(cbHandle)
        return cbHandle

    def authenticate(self, auth_info, use_auth, callback, arg):
        """
              authenticate the user to the FTP server
              **args**
              *auth_info*    a Python object which wraps a globus_auth_info object
              *use_auth*    a boolean which if true GSS authentication will be used, otherwise username, password
              authentication will be used
              *callback*      a Python function that has the signature func(user_arg,handle, error, ftp_response)
              *arg*        the user_arg to the callback function

              *Returns*
               returns a SWIG'ized callback handle

               *Throws*
               throws a FtpControlException if the command could not be sent
              """
        ret, cbHandle = ftpControlc.authenticate(self._handle,auth_info,callback, arg)
        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex

        self._cbList.append(cbHandle)
        return cbHandle
    
    def send_command(self,cmdspec, callback, arg):
        """send a command to the FTP server and register a response handler
              **args**
              *cmdspec* a string which has the command to be sent to the FTP Server
              *callback* a Python function that has the signature func(user_arg,handle, error, ftp_response)
              *arg*        the user_arg to the callback function

              *Returns*
               returns a SWIG'ized callback handle

               *Throws*
               throws a FtpControlException if the command could not be sent

              """
        ret, cbHandle = ftpControlc.send_command(self._handle,cmdspec,callback, arg)
        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex

        self._cbList.append(cbHandle)
        return cbHandle

    def abort(self, callback, arg):
        """send an ABORT to the FTP server and register a response handler
              **args**
              *callback* a Python function that has the signature func(user_arg,handle, error, ftp_response)
              *arg*        the user_arg to the callback function
              *Returns*
               returns a SWIG'ized callback handle

               *Throws*
               throws a FtpControlException if the ABORT message could not be sent
              """
        ret, cbHandle = ftpControlc.py_abort(self._handle,callback, arg)
        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex

        self._cbList.append(cbHandle)
        return cbHandle

    def quit(self, callback, arg):
        """send a QUIT to the FTP server and register a response handler
               **args**
               
               *callback* a Python function that has the signature func(user_arg,handle, error, ftp_response)
              *arg*        the user_arg to the callback function
              *Returns*
               returns a SWIG'ized callback handle

               *Throws*
               throws a FtpControlException if the QUIT message could not be sent

              """
        ret, cbHandle = ftpControlc.py_quit(self._handle,callback, arg)
        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex

        self._cbList.append(cbHandle)
        return cbHandle

    def force_close(self, callback, arg):
        """force a close of the control connection without sending a QUIT or waiting for outstanding commands
              to complete
              **args**
               
               *callback* a Python function that has the signature func(user_arg,handle, error, ftp_response)
              *arg*        the user_arg to the callback function
              *Returns*
               returns a SWIG'ized callback handle

               *Throws*
               throws a FtpControlException if the channel could not be closed
              """
        ret, cbHandle = ftpControlc.force_close(self._handle,callback, arg)
        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex

        self._cbList.append(cbHandle)
        return cbHandle


    def data_connect_read(self, callback, arg):
        """Create an incoming FTP data connection

               **args**
               *callback* a python function that has the signature func(user_arg, handle, error, buffer, length, offset, eof)
              *arg*        the user_arg to the callback function
              **Returns**
               returns a SWIG'ized callback handle
               
               **Throws**
               throws a FtpControlException if the data connection could not be created
              """
        ret, cbHandle = ftpControlc.data_connect_read(self._handle,callback, arg)
        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex

        self._cbList.append(cbHandle)
        return cbHandle

    def data_add_channels(self,num_channels, stripe_ndx):
        """Opens additional data channels (connections) to the host identified by the stripe parameter
              **args**
              *num_channels*  The number of additional channels to add
              *stripe_ndx*  The stripe that the channels will belong to

              **Returns**
              nothing

              **Throws**
               throws a FtpControlException if the channels could not be added
              """
        retString = ftpControlc.data_add_channels(self._handle,num_channels, stripe_ndx)
        if not retString == "NONE":
            ex = FtpControlException(retString)
            raise ex
        return
    def data_send_eof(self, eof_array, eof, callback, arg):
        """Sends an eof message to each stripe along an open data connection
              **args**
              *eof_array* a python argument which wraps an array of integers.
              The array of integers should contain an integer that will be added to the curretn parallel data connection
              count on each stripe. The order of the integers corresponds to th eeach stripe in the same order as what was
              returned from local_port()
              *eof* boolean which is the eof message to be sent to all of the channels specified in the eof array
              *callback* a function with the signature func(user_arg, handle, error)
              *arg* the user_arg for the callback function

              **Returns**
               returns a SWIG'ized callback handle
              
              **Throws**
              throws a FtpControlException if the eofs could not be sent
              """
        ret, cbHandle =  ftpControlc.data_send_eof(self._handle, eof_array.get_handle(), eof_array.get_size(), eof, callback, arg)
        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex

        self._cbList.append(cbHandle)
        return cbHandle

    def data_remove_channels(self, num_channels, stripe_ndx):
        """Removes data channels, to the host identified by the stripe parameter
              **args**
              *num_channels*  The number of additional channels to add
              *stripe_ndx*  The stripe that the channels will belong to

              **Returns**
              nothing

              **Throws**
               throws a FtpControlException if the channels could not be removed

              """
        ret, error = ftpControlc.data_remove_channels(self._handle, num_channels, stripe_ndx)
        if not ret == 0:
            ex = FtpControlException(error)
            raise ex

        return


    def data_query_channels(self, stripe_ndx):
        """Returns the number of currently open channels for the host identified by the stripe parameter
              **args**
              *stripe_ndx*  The stripe that you wish to query

              **Returns**
              number of channels on that stripe

              **Throws**
               throws a FtpControlException if the stripe could not be queried
              """
        ret, num_channels = ftpControlc.data_query_channels(self._handle, stripe_ndx)
        if not ret == 0:
            ex = FtpControlException(num_channels)
            raise ex

        return num_channels

    def get_total_data_channels(self, stripe_ndx):
        """Returns the number of data channels used so far on this particular stripe
               **args**
              *stripe_ndx*  The stripe that you wish to query

              **Returns**
              number of channels on that stripe

              **Throws**
               throws a FtpControlException if the number of channels could not be determined

              """
        ret, num_channels = ftpControlc.data_get_total_data_channels(self._handle, stripe_ndx)
        if not ret == 0:
            ex = FtpControlException(num_channels)
            raise ex

        return num_channels

    def local_send_eof(self, eof):
        """Determines if the library will automatically send an EOF message in extended block mode,
               or if the user will have to explicity do it by calling data_send_eof
               **args**
               *eof* boolean whether to set the handle to have a local eof

               **Returns**
               nothing

               **Throws**
               throws a FtpControlException if the local_eof could not be set
               """
        ret, error = ftpControlc.local_send_eof(self._handle, eof)
        if not ret == 0:
            ex = FtpControlException(error)
            raise ex

        return
    
    def local_parallelism(self, parallelism):
        """Set the parallelism information in a FTP control handle
              **args**
              *parallelism* a ftpControl.Parallelism object

              **Returns**
              nothing
              
              **Raises**
              a FtpControlException is raised if the parallelism could not be set
              """
        ret, error = ftpControlc.local_parallelism(self._handle, parallelism.get_handle())
        if not ret == 0:
            ex = FtpControlException(error)
            raise ex

        return

    def local_spas(self, address_array):
        """Create num_addresses local listening socketes, bind them and return the addresses the sockets are listening to

              **args**
              *address_array* a Python object that wraps an array of  host_port_t objects

              **Returns**
              the address_array object

              **Raises**
              a FtpControlException is raised if the local_spas cannot be set

              *****************************************************************************************************************************************
              As of the globus 2.0 final release this function is not implemented, so an unimplemented exception will
              always be thrown
              MKR
              6/17/02
              *****************************************************************************************************************************************
              """
        ret, address_handle = ftpControlc.local_spas(self._handle, address_array.get_handle(), address_array.get_size())
        if not ret == 0:
            ex = FtpControlException(address_handle)
            raise ex 

        host_array = HostPortArray(address_handle)
        return host_array

    def local_spor(self,address_array):
        """Insert the host/port addresses returned by a SPAS on the remote host into the local FTP control handle.

              **args**
              *address_array* a Python object that wraps an array of  host_port_t objects

              **Returns**
              the address_array object

               **Raises**
              a FtpControlException is raised if the local_spor cannot be set

              *****************************************************************************************************************************************
              As of the globus 2.0 final release this function calls the globus_ftp_control_local_spas function which is not implemented, so
              an unimplemented exception will always be thrown
              MKR
              6/17/02
              *****************************************************************************************************************************************
              """
        
        ret, address_handle = ftpControlc.local_spor(self._handle, address_array.get_handle(), address_array.get_size())
        if not ret == 0:
            ex = FtpControlException(address_handle)
            raise ex
        
        host_array = HostPortArray(address_handle)
        return host_array

    def local_mode(self, mode):
        """
              Update the ftp handle with the given mode information
              **args**
              *mode*  a FtpControl Mode type

              **Returns**
              nothing

              **Raises**
              a FtpCoontrolException is raised if the local_mode cannot be set
              """
        ret, error = ftpControlc.local_mode(self._handle, mode)
        if not ret == 0:
            ex = FtpControlException(error)
            raise ex

        return

    def local_tcp_buffer(self, tcp_buffer):
        """
              Update the ftp handle with the given socket buffer size information
              **args**
              *tcp_buffer* a Python TcpBuffer Object

              **Returns**
              nothing

              **Raises**
              a FtpCoontrolException is raised if the local_tcp_buffer cannot be set

              The TcpBuffer.set_fixed call must be made on the tcp_buffer object before this call is made
              """
        ret, error = ftpControlc.local_tcp_buffer(self._handle, tcp_buffer.get_handle())
        if not ret == 0:
            ex = FtpControlException(error)
            raise ex

        return

    def local_dcau(self, dcau, credential):
        """
              Update the ftp handle with the given data channel authentication information
              **args**

              *dcau*  a Python DCAU

              *credential* a Python GSSCred object

              **Returns**
              nothing

              **Raises**
              a FtpControlException is raised if the local_dcau cannot be set
              """
        ret,error = fcpControlc.local_dcau(self._handle, dcau.get_handle(), credential.get_handle())
        if not ret == 0:
            ex = FtpControlException(error)
            raise ex
        return

    def local_pbsz(self, size):
        """
              Update the ftp handle with the given protection buffer size information
              **args**

              *size* the size that the buffer will be during a protected transfer
              
              **Returns**
              nothing

              **Raises**
               a FtpControlException is raised if the local_pbsz cannot be set
              """

        
        ret, error = ftpControlc.local_pbsz(self._handle, size)
        if not ret == 0:
            ex = FtpControlException(error)
            raise ex
        return

    def get_pbsz(self):
        """
              Return the size of the protection buffer size information

              **Returns**
              the size of the buffer during protected transfers

              **Raises**
              a FtpControlException is raised if the pbsz cannot be retrieved
              """
        ret, size = ftpControlc.get_pbsz(self._handle)
        if not ret == 0:
            ex = FtpControlException(error)
            raise ex
        return size
 
    def local_stru(self, structure):
        """Update the handle information on the structure of the data being sent on the data channel"""
        ret, error = ftpControlc.local_stru(self._handle, structure.get_handle())
        if not ret == 0:
            ex = FtpControlException(error)
            raise ex
        return

    def data_read(self, buffer, callback, arg):
        """Reads data from data connection(s) and puts it in the specified buffer"""

        ret, cbHandle = ftpControl.data_read(self._handle, buffer.get_handle(), buffer.get_size(), callback, arg)
        if not ret == 0:
            ex = FtpControlException(cbHandle)
            raise ex
        
        self._cbList.append(cbHandle)
        return cbHandle

class HostPortArray:
    """Class that wraps an array of host_port """

    def __init__(self,size, handle = None):
        self._activated = 0
        self._size = size
        ret = utilc.globus_module_activate(ftpControlc.get_module())
        if not ret == 0:
            ex = HostPortException("Unable to initialize the module")
            raise ex
        self._activated = 1
        if handle == None:
            handle = ftpControlc.host_port_array_init(size)
        self._handle = handle    
        
    def __del__(self):
        ftpControlc.host_port_array_destroy(self._handle)

    def get_handle(self):
        return self._handle

    def get_size(self):
        return self._size

class HostPort:
    """Class that wraps the host_port_t struct"""

    def __init__(self,host, port, handle = None):
        self._activated = 0
        ret = utilc.globus_module_activate(ftpControlc.get_module())
        if not ret == 0:
            ex = HostPortException("Unable to initialize the module")
            raise ex
        self._activated = 1
        if handle == None:
            handle = FtpControlc.host_port_init(host, port)
            
        self._handle = handle

    def __del__(self):
        FtpControlc.host_port_destroy(self._handle)

    def get_handle(self):
        return self._handle
    
class EofArray:
    """This class wraps an array of integers; an instance of this class should be created to use FtpControl.data_send_eof()"""
    def __init__(self, size):
        self._activated = 0
        ret = utilc.globus_module_activate(ftpControlc.get_module())
        if not ret == 0:
            ex = EofArrayException("Unable to initialize the module")
            raise ex
        self._activated = 1
        self._size = size
        handle = FtpControlc.eof_array_init(size)
        self._handle = handle

    def __del__(self):
        FtpControlc.eof_array_destroy(self._handle)

    def get_handle(self):
        return self._handle
    
class Command:
    
    def __init__(self, raw_command, auth_info, handle = None):
        self._activated = 0
        ret = utilc.globus_module_activate(ftpControlc.get_module())
        if not ret == 0:
             ex = CommandException("Unable to initialize the module")
             raise ex
        self._activated = 1
        if handle == None:
            handle = ftpControlc.command_init(raw_command, auth_info.get_handle())
        self._handle = handle
        return

    def __del__(self):
        ftpControlc.command_destroy(self._handle)
        ret =  utilc.globus_module_deactivate(ftpControlc.get_module())
        if not ret == 0:
            ex = CommandException("Unable to deactivate the module")
            raise ex
        return

    def copy(self):
        ret, new_handle = ftpControlc.command_copy(self._handle)
        if not ret == 0:
            ex = CommandException(new_handle)
            raise ex
        com = Command(new_handle)
        return com
    
class DCAU:

    def __init__(self, handle = None):
        self._activated = 0
        ret = utilc.globus_module_activate(ftpControlc.get_module())
        if not ret == 0:
             ex = DCAUException("Unable to initialize the module")
             raise ex
        self._activated = 1
        if handle == None:
            handle = ftpControlc.dcau_init()
        self._handle = handle
        return

    def __del__(self):
        ftpControlc.dcau_destroy(self._handle)
        ret = utilc.globus_module_deactivate(ftpControlc.get_module())
        if not ret == 0:
            ex = DCAUException("Unable to deactivate the module")
            raise ex
        return

    def set_mode(self, mode):
        ret = ftpControlc.dcau_set_mode(self._handle, mode)
        if not ret == 0:
            ex = DCAUException("Unable to set the mode")
            raise ex
        return

    def set_subject(self, subject):
        ret = ftpControlc.dcau_set_subject(self._handle, subject)
        if not ret == 0:
            ex = DCAUException("unable to set the subject")
            raise ex
        return
    
    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed.
    #######################################################################
    def get_handle(self):
        return self._handle

class Layout:

    def __init__(self, handle = None):
        self._activated = 0
        ret = utilc.globus_module_activate(ftpControlc.get_module())
        if not ret == 0:
            ex = LayoutException("Unable to initialize the module")
            raise ex
        self._activated = 1
        if handle == None:
            handle = ftpControlc.layout_init()
        self._handle = handle
        return

    def __del__(self):
        ftpControlc.layout_destroy(self._handle)
        ret = utilc.globus_module_deactivate(ftpControlc.get_module())
        if not ret == 0:
            ex = LayoutException("Unable to deactivate the module")
            raise ex
        return

    def set_striping_mode(self, mode):
        ret = ftpControlc.layout_set_striping_mode(self._handle, mode)
        if ret == -42:
            ex = LayoutException("unknown striping mode")
            raise ex
        if not ret == 0:
            ex = LayoutException("unable to set the striping mode")
            raise ex
        return

    ######################################################################
    # XXX This call is only useful in round robin mode.  Also on 64bit
    # platoforms size should be a python long.
    #####################################################################
    def set_round_robin_block_size(self, size):
        ftpControlc.layout.set_round_robin_block_size(self._handlle,
                                                     size)
        return

    #######################################################################
    # XXX This call is only useful in partitioned mode.  Also the size
    # argument should be a python long on 64 bit machines.
    ######################################################################
    def set_partition_block_size(self, size):
        ftpControlc.layout.set_partition_block_size(self._handle, size)
        return
    
    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed.
    #######################################################################
    def get_handle(self):
        return self._handle


class Response:

    def __init__(self, handle = None):
        """hmm I wonder how one of these structs are created"""
        pass

    def __del__(self):
        retString = ftpControlc.response_destroy(self._handle)
        if not retString == "NONE":
            ex = ResponseException(retString)
            raise ex
        

    def copy(self):
        ret, response_copy = ftpControl.response_copy(self._handle)
        if not ret == 0:
            ex = ResponseException(response_copy)
            raise ex

        return response_copy

class AuthInfo:
    """Class that wraps auth_info_t struct"""
    def __init__(self, handle, credential, encrypt, user, password, account, subject):
        ret, handle = ftpControlc.auth_info_init(credential,encrypt, user, password, account, subject)
        if not ret == 0:
            ex = AuthInfoException(handle)
            raise ex
        self._handle = handle
        return

    def get_handle(self):
        return self._handle

    def compare(self, auth_info):
        """Returns true if the AuthInfo objects are equivelent otherwise returns false"""
        ret = ftpControlc.auth_info_compare(self._handle, auth_info._handle)
        if ret == 0:
            return 1
        else:
            return 0
        
class Parallelism:

    def __init__(self, handle = None):
        self._activated = 0
        ret = utilc.globus_module_activate(ftpControlc.get_module())
        if not ret == 0:
            ex = ParallelismException("Unable to initialize the module")
            raise ex
        self._activated = 1
        if handle == None:
            handle = ftpControlc.parallelism_init()
        self._handle = handle
        return

    def __del__(self):
        ftpControlc.parallelism_destroy(self._handle)
        ret = utilc.globus_module_deactivate(ftpControlc.get_module())
        if not ret == 0:
            ex = ParallelismException("Unable to deactivate the module")
            raise ex
        return

    def set_size(self, size):
        ret = ftpControlc.parallelism_set_size(self._handle, size)
        if not ret == 0:
            ex = ParallelismException("unable to set the size")
            raise ex
        return

    def set_mode(self, mode):
        ret = ftpControlc.parallelism_set_mode(self._handle, mode)
        if ret == -42:
            ex = ParallelismException("unknown mode")
            raise ex
        if not ret == 0:
            ex = ParallelismException("unable to set the mode")
            raise ex
        return

    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed.
    #######################################################################
    def get_handle(self):
        return self._handle

class FtpControlServer:

    def __init__(self, handle = None):
        self._activated = 0
        ret = utilc.globus_module_activate(ftpControlc.get_module())
        if not ret == 0:
            ex = FtpControlException("Unable to initialize the module")
            raise ex
        self._cbList = []
	self._activated = 1
        if handle == None:
            ret, handle = ftpControlc.server_handle_init()
        if not ret == 0:
            ex = FtpControlException("unable to create the handle")
            raise ex
        self._handle = handle
        return
    
    def __del__(self):
        for cb in self._cbList:
            self.free_callback(cb)
        if not self._handle == None:
            ret = ftpControlc.server_handle_destroy(self._handle)
        if not ret == 0:
            ex = FtpControlException("unable to destroy the control handle")
            raise ex
        ret = utilc.globus_module_deactivate(ftpControlc.get_module())
        if not ret == 0:
            ex = FtpControlException("Unable to deactivate the module")
            raise ex
        return

    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed.
    #######################################################################
    def get_handle(self):
        return self._handle

    def accept(self, handle, callback, arg):
        print "...accpet start"
        ret, cbHandle = ftpControlc.server_accept( self._handle, handle.get_handle(), callback, arg)
        if not ret == 0:
            ex = FtpControlExecption(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        print "...accpet end"
        return cbHandle

    def doit(self):
        print "in doit"

    def print_hello(self):
        print "...in print hello FtpControlServer.py"
        ftpControlc.print_hello()
        print "...finished print hello FtpControlServer.py"

    def auth(self, handle, auth_req, callback, arg):
        print "in auth"
        ret, cbHandle = ftpControlc.server_authenticate(handle.get_handle(), auth_req, callback, arg)
        if not ret == 0:
            ex = FtpControlExecption(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        print "end auth"
        return cbHandle

    def oldauthenticate(self, auth_req, callback, arg):
        print ""
        print "in authenticated"
        ret, cbHandle = ftpControlc.server_authenticate( self._handle, auth_req,	callback, arg)
        print "middle authenticated"
        if not ret == 0:
            ex = FtpControlExecption(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        print "end authenticated"
        return cbHandle

    def doit2(self):
	print "in doit2"

	################################################################
	# This is a strange function -- it used to take a pointer to
	# to the port, and modify it to the port that was assigned to
	# the listener. I now just return the assigened port back to the
	# calling funtion 
	#
	################################################################

    def listen(self, port, callback, arg):
        print "listen start"
        ret, cbHandle, real_port = ftpControlc.server_listen( self._handle, port, callback, arg)
        if not ret == 0:
            ex = FtpControlExecption(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        print "listen end"
        return cbHandle, port

    def stop(self, callback, arg):
        ret, cbHandle = ftpControlc.server_stop( self._handle, callback, arg)
        if not ret == 0:
            ex = FtpControlExecption(cbHandle)
            raise ex
        self._cbList.append(cbHandle)
        return cbHandle

class TcpBuffer:

    def __init__(self, handle = None):
        self._activated = 0
        ret = utilc.globus_module_activate(ftpControlc.get_module())
        if not ret == 0:
            ex = TcpBufferException("Unable to initialize the module")
            raise ex
        self._activated = 1
        if handle == None:
            handle = ftpControlc.tcpbuffer_init()
        self._handle = handle
        return

    def __del__(self):
        ftpControlc.tcpbuffer_destroy(self._handle)
        ret = utilc.globus_module_deactivate(ftpControlc.get_module())
        if not ret == 0:
            ex = TcpBufferException("Unable to deactivate the module")
            raise ex
        return

    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed.
    #######################################################################
    def get_handle(self):
        return self._handle
    
    def set_automatic(self, initial, minimum, maximum):
        ftpControlc.tcpbuffer_set_automatic(self._handle, initial,
                                 minimum, maximum)
        return

    def set_fixed(self, size):
        ftpControlc.tcpbuffer_set_fixed(self._handle, size)
        return

    def set_default(self):
        ftpControlc.tcpbuffer_set_default(self._handle)
        return

    
