############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
"""
Provides an interface to the gass file module.
"""

from pyGlobus import utilc
from pyGlobus.util import GlobusException
import gassFilec

class GassFileException(GlobusException):
    """
    A simple subclass of GlobusException.
    
    Used for errors in the gass file module. It defines no new methods.
    
    **Extends:**
     
         *pyGlobus.util.GlobusException*
    """
    pass

class GassFile:
    """
    Provides an interface to the gass file module.

    Allows the opening and closing of remote files using GASS to provide location
    independence. The files can either be manipulated as python file objects, or
    through the os module as file descriptors.
    """
    
    def __init__(self):
        """
        Constructs an instance.

        Activates the underlying globus gass file module.

        **Raises:**

            A GassFileException is thrown if unable to initialize the module.
        """
        self.activated = 0
        ret = utilc.globus_module_activate(gassFilec.get_module())
        if not ret == 0:
            ex = GassFileException("Unable to activate the module")
            raise ex
        self.activated = 1

    def __del__(self):
        """
        Destroy an instance.

        Deactivates the underlying module.

        **Raises:**

            A GassFileException is thrown if unable to deactivate the module.
        """
        if self.activated:
            ret = utilc.globus_module_deactivate(gassFilec.get_module())
            if not ret == 0:
                ex = GassFileException("Unable to deactivate the module")
                raise ex

    def open(self, file, oflags):
        """
        Open the given file.

        Similar to the unix open call, except the filename argument should be a gass url.

        **Arguments:**

            - *file* A url for the file to be open. Currently http, https, ftp, gsiftp are
            supported.

            - *flags* The standard unix open flags. They're defined in the os module.

        **Raises:**

            A GassFileException is thrown if unable to open the url.

        **Returns:**

            An integer file descriptor suitable for use with the os module
            is returned.
        """
        ret = gassFilec.gass_open(file, oflags)
        if ret == -1:
            ex = GassFileException("Unable to open %s" % file)
            raise ex
        return ret

    def close(self, fd):
        """
        Close the given file descriptor.

        Similar to the unix close call, except it should only be used to close file
        descriptors returned from the open method.

        **Arguments:**

            - *fd* The file descriptor to be closed.

        **Raises:**

            A GassFileException is thrown if unable to close the file.
        """
        ret = gassFilec.gass_close(fd)
        if ret == -1:
            ex = GassFileException("Error while closing the file")
            raise ex
        return

    def fopen(self, file, mode):
        """
        Open the given file as a python file object.

        Similar to the unix fopen call, except the file name should be a url.

        **Arguments:**

            - *file* The gass url to be opened. Currently http, https, ftp, gsiftp
            and local files are supported.

            - *mode* A string containing the mode to open the file in. The modes
            are the same as the modes used in the unix fopen call.

        **Raises:**

            A GassFileException is thrown if unable to open the file.

        **Returns:**

            A python file object connected to the gass file is returned.
        """
        ret = gassFilec.gass_fopen(file, mode)
        if ret == -1:
            ex = GassFileException("Unable to fopen %s" % file)
            raise ex
        return ret

    def fclose(self, file):
        """
        Close the give python file object.

        Similar to the unix fclose call, except it should only be used to close a file
        opened with the fopen method.

        **Arguments:**

            - *file* The python file object, returned fro the fopen method, to be
            closed.

        **Raises:**

            A GassFileException is thrown if unable to close the file.
        """
        ret = gassFilec.gass_fclose(file)
        if ret == -1:
            ex = GassFileException("Error while trying to fclose the file")
            raise ex
        return
