############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
"""
The main interface to the Globus Gram Client functionality.
"""

from pyGlobus import utilc
from pyGlobus.util import GlobusException
import gramClientc

class GramClientException(GlobusException):
    """
    A simple subclass of GlobusException.

    Used for errors in the gram client module. It defines no new methods.

    **Extends:**
    
        *pyGlobus.util.GlobusException*
    """
    pass

##############################################################################
# A set of module constants used in the submit_request call.  They describe
# what state changes your interested in being informed of.  They can be
# bitwise or'd together.
#############################################################################
JOB_STATE_PENDING   = 1
JOB_STATE_ACTIVE    = 2
JOB_STATE_FAILED    = 4
JOB_STATE_DONE      = 8
JOB_STATE_SUSPENDED = 16
JOB_STATE_ALL       = 31

class GramClient:
    """
    The main interface to the gram client module.

    It is mainly a proxy class for the gramClient module. It provides a higher
    level interface designed to simplify usage.  Exceptions are used to indicate
    all error conditions.
    """
    
    def __init__(self):
        """
        Constructs an instance.
        
        This will activate the globus_gram_client_mddule.

        **Raises:**
        
            A GramClientException is thrown if unable to activate the module.
        """
        self._activated = 0
        self._cbDict = {}
        ret = utilc.globus_module_activate(gramClientc.get_module())
        if not ret == 0:
            ex = GramClientException("Unable to initialize the module")
            raise ex
        self._activated = 1

    def __del__(self):
        """
        Destroys an instance.

        It will deactivate the globus_gram_client module when
        the object is destroyed.

        **Raises:**
        
            A GramClientException is thrown if unable to destroy the module. This
            exception can't be caught, and should never happend.  It's there mainly
            for debugging if it does happen. :-)
        """
        for cb in self._cbDict.keys():
            try:
                self.remove_callback(cb)
            except GramClientException:
                pass
        if self._activated:
            ret = utilc.globus_module_deactivate(gramClientc.get_module())
            if not ret == 0:
                ex = GramClientException("Unable to deactivate the module")
                raise ex

    def ping(self, jobContact):
        """
        Pings a gatekeeper.

        **Arguments:**
        
            - *jobContact* A string containing the contact information
        """
        status = gramClientc.ping(jobContact)
        if not status == 0:
            error = gramClientc.globus_gram_protocol_error_string(status)
            raise GramClientException(error)
        return
    
    def set_callback(self, callback, arg):
        """
        Sets a callback.

        Sets the function to be notified of job state changes.

        **Arguments:**
        
            - *callback* The function that should be called when the job state changes.
            The function has the following prototype:
            'function(arg, contact, state, error)'
            where contact is the job contact, state is one of the module constants
            defined above, and error is an integer error code.  It can be passed to
            the error_string function to return an error string.

            - *arg* The argument to be passed to the callback when it is called.

        **Raises:**
        
            A GramClientException if an error occurs while setting the callback.

        **Returns:**
        
            A string containing the callback contact is returned. This can
            be passed to submit_request.
        """
        status, handle, contact = gramClientc.callback_allow(callback, arg)
        if not status == 0:
            error = gramClientc.globus_gram_protocol_error_string(status)
            ex = GramClientException(error)
            raise ex
        self._cbDict[contact] = handle
        return contact

    def submit_request(self, resourceManager, description, jobStateMask, callbackContact):
        """
        Submits a job request to a resource manger.

        **Arguments:**
        
            - *resourceManager* The resource manager to submit the job to.

            - *description* An RSL string that contains the job description.

            - *jobStateMask* Which job state change message you wish to be notified of.
            The notification happens through the callback. This is an int that can
            be formed by bitwise or'ing the module constants defined above.

            - *callbackContact* A string containing the contact for the callback that
            should be notified of job state changes.

        **Raises:**
        
            A GramClientException if unable to submit the job request.

        **Returns:**
        
            A string containing the job contact used for communicting with the
            remote job.
        """
        status, contact = gramClientc.job_request(resourceManager, description,
                                                 jobStateMask, callbackContact)
        if not status == 0:
            error = gramClientc.globus_gram_protocol_error_string(status)
            ex = GramClientException(error)
            raise ex
        return contact

    def check_status(self, jobContact):
        """
        Checks the status of a current job.

        **Arguments:**
        
            - *jobContact* A string containing the contact information for the job
            you would like the status of.

        **Raises:**
        
            A GramClientException if unable to get the job status.

        **Returns:**
        
            An int containing the current job status.  This int will be one
            of the module constants defined above.
        """
        ret, status, failure = gramClientc.job_status(jobContact)
        if not ret == 0:
            error = gramClientc.globus_gram_protocol_error_string(failure)
            ex = GramClientException(error)
            raise ex
        return status

    def cancel_job(self, jobContact):
        """
        Cancels a job.

        **Arguments:**
        
            - *jobContact* A string containing the contact information for the job to
            be canceled.

        **Raises:**
        
            A GramClientException if unable to cancel the job.
        """
        status = gramClientc.job_cancel(jobContact)
        if not status == 0:
            error = gramClientc.globus_gram_protocol_error_string(status)
            ex = GramClientException(error)
            raise ex
        return

    def remove_callback(self, callbackContact):
        """
        Stops the callback.

        Stops the callback from listening for job state changes and cleans up
        any resources.

        **Arguments:**
        
            - *callbackContact* The string identifier for the callbck to be removed.

        **Raises:**
        
            A GramClientException if unable to remove the callback.
        """
        try:
            handle = self._cbDict[callbackContact]
        except KeyError:
            ex = GramClientException("Unable to find the callback contact")
            raise ex
        status = gramClientc.remove_callback(callbackContact, handle)
        if not status == 0:
            error = gramClientc.globus_gram_protocol_error_string(status)
            ex = GramClientException(error)
            raise ex
        del self._cbDict[callbackContact]
        return

    def register_callback(self, jobContact, stateMask, callbackContact):
        """
        Registers a callback.

        Sets the functions to receive job state changes for the given job.

        **Arguments:**
        
            - *jobContact* The job who's changes should be reported to the given
            callback.
             
            - *stateMask* Which job state change messages you wish to be notified of.
            The notification happens through the callback. This is an int that
            can formed by bitwise or'ing the module constants defined above.

            - *callbackContact* The callback that should be notified.

        **Raises:**
        
            A GramClientException if unable to register the callback.

        **Returns:**
        
            An int containing the current job status. This is one of the
            values defined in the module constants.
        """
        ret, status, failure = gramClientc.callback_register(jobContact, stateMask,
                                                            callbackContact)
        if not ret == 0:
            error = gramClientc.globus_gram_protocol_error_string(failure)
            ex = GramClientException(error)
            raise ex
        return status

    def unregister_callback(self, jobContact, callbackContact):
        """
        Unregisters a callback.

        Stop notifing the callback of job state changes for the given job.

        **Arguments:**
        
            - *callbcackContact* The callback that should no longer be notified.

        **Raises:**
        
            A GramClientException is thrown if unable to unregister the
            callback.

        Returns:
        
            An int containing the current job status.  This is one of the
            values defined in the module constants.
        """
        ret, status, failure = gramClientc.callback_unregister(jobContact,
                                                              callbackContact)
        if not ret == 0:
            error = gramClientc.globus_gram_protocol_error_string(failure)
            ex = GramClientException(error)
            raise ex
        return status

