############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
"""
The main interface to the Replica Catalog functionality.
"""

from pyGlobus import utilc
from pyGlobus.util import GlobusException
import replicaCatalogc

#############################################################
# Module Functions
############################################################

def create(url, authMode, filenames=None,
           objectclass=replicaCatalogc.COLLECTION_OBJECTCLASS,
           attr=None, bindDN=None, passwd=None):
    """
    Create a new collection.

    Creates a new collection based on the given url, connection attributes and objectclass.
    Also associates the given filenames with the collection.

    The collection must not already exist.
    
    **Arguments**

        - *url* The ldap url of the collection to create.

        - *authMode* The authentication mode to use when creating the
        collection. If the mode is secure, bindDN and passwd should be None.
 
        - *filenames* A list of filenames to add to the collection. If None,
        a empty collection is created.

        - *objectclass*  objectclass name of the collection to be created, must
        be a subclass of COLLECTION_OBJECTCLASS, if None is given then the default
        COLLECTION_OBJECTCLASS is used.

        - *attr* A CollectionAttr object containing the attributes for the collection.
    
        - *bindDN* String name to use when authenticating to the ldap server.

        - *passwd* String password to authenticate with.
    
        **Raises:**
        
        A CollectionException is thrown if unable to activate the module, or
        create the connection.
    """
    ret = utilc.globus_module_activate(replicaCatalogc.get_module())
    if not ret == 0:
        ex = CollectionException("Unable to initialize the module")
        raise ex
    if attr == None:
        ret, attrPtr = replicaCatalogc.handleattr_init()
        if not ret == 0:
            ex = CollectionException(attrPtr)
            raise ex
        attr = CollectionAttr(attrPtr)
    ret, err = replicaCatalogc.set_authentication_mode(attrPtr, authMode,
                                                       bindDN, passwd)
    if not ret == 0:
        ex = CollectionException(err)
        raise ex
    ret, handle = replicaCatalogc.collection_create(attrPtr, url, filenames,
                                                    objectclass)
    if not ret == 0:
        ex = CollectionException(handle)
        raise ex
    return Collection("", None, attr, None, None, handle, 1)

##########################################################
# Module Classes
##########################################################

class AttributeSetException(GlobusException):
    """
    A simple subclass of GlobusException.

    Used for errors in the replica catalog attribute set module. It defines no new methods.

    **Extends:**
    
        *pyGlobus.util.GlobusException*
    """
    pass

class CollectionException(GlobusException):
    """
    A simple subclass of GlobusException.

    Used for errors in the replica catalog collection module. It defines no new methods.

    **Extends:**
    
        *pyGlobus.util.GlobusException*
    """
    pass

class CollectionAttrException(GlobusException):
    """
    A simple subclass of GlobusException.

    Used for errors in the replica catalog collection attribute module. It defines no new methods.

    **Extends:**
    
        *pyGlobus.util.GlobusException*
    """
    pass

class EntrySetException(GlobusException):
    """
    A simple subclass of GlobusException.

    Used for errors in the replica catalog entry set module. It defines no new methods.

    **Extends:**
    
        *pyGlobus.util.GlobusException*
    """
    pass

class AttributeSet:
    """
    The main interface to the replica catatlog attribute set module.

    Manages access to the attribute set, and hides the underlying memory
    management.
    """
    
    def __init__(self, handle):
        """
        Construct an instance.
        
        This will construct an instance from a given handle. This
        constructor. is for module use, and should not be called. Instead
        calls like EntrySet.get_attributes() returns an attribute set instance.

        **Arguments**

            - *handle* A SWIG'ized pointer to a globus_replica_catalog_collection_attribute_set.
        """
        self._handle = handle
        return

    def __del__(self):
        """
        Destroys an instance.

        It will destroy the attribute set, and free the underlying memory.

        **Raises:**
        
            A AttributeSetException is thrown if unable to destroy the attribute set. This
            exception can't be caught, and should never happend.  It's there mainly
            for debugging if it does happen. :-)
        """
        if not self._handle == None:
            ret, err = replicaCatalogc.attribute_set_destroy(self._handle)
            if not ret == 0:
                ex = AttributeSetException(err)
                raise ex
        return

    def __getitem__(self, index):
        """
        Limited sequence operator.

        Allows normal python list iteration. Returns a 2-tuple. The first
        element is the attribute name, and the second is a list of values.
        """
        if index == 0:
            self.first()
        if not self.more():
            raise IndexException
        name = self.get_name()
        values = self.get_values()
        self.next()
        return (name, values)
    
    def first(self):
        """
        Set the current attribute to the first attribute in the set.

        **Raises**

            An AttributeSetException is thrown if unable to reset the current
            attribute.
        """
        ret, err = replicaCatalogc.attribute_set_first(self._handle)
        if not ret == 0:
            ex = AttributeSetException(err)
            raise ex
        return

    def next(self):
        """
        Set the current attribute to the next attribute in the set.

        Must be preceded by a call to first.
        
        **Raises**

            An AttributeSetException is thrown if unable to reset the current
            attribute.
        """
        ret, err = replicaCatalogc.attribute_set_next(self._handle)
        if not ret == 0:
            ex = AttributeSetException(err)
            raise ex
        return

    def more(self):
        """
        Whether there are more attributes in an attribute set, inclusive of the current attribute.

        Must be preceded by a call to first.

        **Returns**

            Returns true if there are more attributes, and false if none.

        **Raises**

            An AttributeSetException is thrown if an error occurs.
        """
        isMore = replicaCatalogc.globus_replica_catalog_attribute_set_more(self._handle)
        return isMore

    def get_name(self):
        """
        Get the name of the current attribute in an attribute set.

        Must be preceded by a call to first.

        **Retuns**

            A string is returned containing the name of the current
            attribute.

        **Raises**

            An AttributeSetException is thrown if unable to get the name.
        """
        ret, name = replicaCatalogc.attribute_set_get_name(self._handle)
        if not ret == 0:
            ex = AttributeSetException(name)
            raise ex
        return name

    def get_values(self):
        """
        Get the values of the current attribute in an attribute set.

        Must be preceded by a call to first.

        **Retuns**

            A list of strings is returned containing the values of the current
            attribute.

        **Raises**

            An AttributeSetException is thrown if unable to get the values.
        """
        ret, values = replicaCatalogc.attribute_set_get_values(self._handle)
        if not ret == 0:
            ex = AttributeSetException(values)
            raise ex
        return values
    
    def get_handle(self):
        """
        Return the underlying replica catalog attribute set handle.

        **XXX** This handle is only valid as long as this object exists.  When
        the object is destroyed, the memory this handle points to is freed.

        **Returns:**

            A string containing the pointer.
        """
        return self._handle

class Collection:
    """
    The main interface to the replica catatlog collection module.

    It is mainly a proxy class for the replicaCatalogc module collection functions. It provides a higher
    level interface designed to simplify usage.  Exceptions are used to indicate
    all error conditions.
    """
    
    def __init__(self, url, authMode, attr=None,
                 bindDN=None, passwd=None, handle=None, activated=0):
        """
        Open a connection to a given collection.
        
        This will activate the globus_replica_catalog_module, and open a connection to
        the collection at the given url.

        **Arguments**

            - *url* The ldap url of the collection to connect to.

            - *authMode* The authentication mode to use when connecting to the
            collection. If the mode is secure, bindDN and passwd should be None.
            
            - *attr* CollectionAttr object containg the attributes for the collection
            handle. If None, default attributes will be used.

            - *bindDN* A string containing the DN to authenticate as to the ldap server
            when doing cleartext authentication.

            - *passwd* A string containing the password to use when authenticating to the
            ldap server when doing cleartext authentication.

            - *handle* A SWIG'ized pointer to a globus_replica_catalog_collection_handle.
            This should only be used by create.

            - *activated* A boolean indicating if the module has already been activated.
            Used by create.
        
        **Raises:**
        
            A CollectionException is thrown if unable to activate the module, or
            open the connection.
        """
        self._activated = activated
        self._handle = None
        ret = utilc.globus_module_activate(replicaCatalogc.get_module())
        if not ret == 0:
            ex = CollectionException("Unable to initialize the module")
            raise ex
        self._activated = 1
        if attr == None:
            ret, attrPtr = replicaCatalogc.handleattr_init()
            if not ret == 0:
                ex = CollectionException(attrPtr)
                raise ex
            attr = CollectionAttr(attrPtr)
        self._attr = attr
        if handle == None:
            ret, err = replicaCatalogc.set_authentication_mode(self._attr._handle,
                                                               authMode, bindDN, passwd)
            if not ret == 0:
                ex = CollectionException(err)
                raise ex
            ret, handle = replicaCatalogc.collection_open(attr._handle, url)
            if not ret == 0:
                ex = CollectionException(handle)
                raise ex
        self._handle = handle
        return

    def __del__(self):
        """
        Destroys an instance.

        It will close the connection, and deactivate the globus_replica_catalog
        module when the object is destroyed.

        **Raises:**
        
            A CollectionException is thrown if unable to destroy the module. This
            exception can't be caught, and should never happend.  It's there mainly
            for debugging if it does happen. :-)
        """
        if not self._handle == None:
            ret, err = replicaCatalogc.collection_close(self._handle)
            if not ret == 0:
                ex = CollectionException(err)
                utilc.globus_module_deactivate(replicaCatalogc.get_module())
                raise ex
        if self._activated:
            ret = utilc.globus_module_deactivate(replicaCatalogc.get_module())
            if not ret == 0:
                ex = CollectionException("Unable to deactivate the module")
                raise ex
        return

    def delete(self):
        """
        Delete a collection and close the connection.

        There must be no locations in the collection. This method does not check for
        the presence of logical files in the collection.

        **Raises**

            A CollectionException is thrown if unable to delete the collection.
        """
        if not self._handle == None:
            ret, err = replicaCatalogc.collection_delete(self._handle)
            if not ret == 0:
                ex = CollectionException(err)
                raise ex
            self._handle = None
        return

    def set_dn(self, dn):
        """
        Set the DN of the collection.

        All subsequent operations will be done with respect to the given DN.

        **Arguments**

            - *dn* String containing the DN to set for the collection.

        **Raises**

            A CollectionException is thrown if unable to set the DN.
        """
        ret, err = replicaCatatlogc.collection_set_dn(self._handle, dn)
        if not ret == 0:
            ex = CollectionException(err)
            raise ex
        return

    def list_collections(self, attrNames=None):
        """
        List the collections in a replica catalog.

        For each collection only return attribute information for the given attribute names.

        **Arguments**

            - *attrNames* A list of names of the attributes to return. If None, the
            default attributes are returned.
        
        **Returns**

            An EntrySet object is returned containing the collections

        **Raises*

            A CollectionException is thrown if unable to return the
            collection list.
        """
        ret, set = replicaCatalogc.list_collections(self._handle, attrNames)
        if not ret == 0:
            ex = CollectionException(set)
            raise ex
        return EntrySet.EntrySet(set)

    def list_rollback_entries(self, url, attr=None, attrNames=None):
        """
        List the rollback and restart information entries in replica catalog.

        **Arguments**

            - *url* URL of the base to begin searching from.
        
            - *attr* A CollectionAttr object containing the attributes of the
            implied connection. If None, default attributes are used.

            - *attrNames* A list of names of the attributes to return. If None, the
            default attributes are returned.

        **Retuns**

            An EntrySet object is returned containing the rollback and
            restart entries.

        **Raises**

            A CollectionException is thrown if unable to return the rollback
            and restart entries.
        """
        ret, set = replicaCatalogc.list_rollback_entries(url, attr._handle, attrNames)
        if not ret == 0:
            ex = CollectionException(set)
            raise ex
        return EntrySet.EntrySet(set)

    def add_filenames(self, filenames, add=0):
        """
        Add filenames to a collection.

        If some of the given filenames already exist in the collection,
        and add is add is true, then add those that don't. Otherwise,
        fail if some of the given filenames already exist and add is
        false.

        **Arguments**

            - *filenames* A list of string filenames to add.

            - *add*     whether to add given filenames that
            don't exist in the collection even if there are some
            given filenames that do exist

        **Raises**

            A CollectionException is thrown if unable to add
            the filenames.
        """
        ret, err = replicaCatalogc.collection_add_filenames(self._handle,
                                                            filenames, add)
        if not ret == 0:
            ex = CollectionException(ex)
            raise ex
        return

    def add_attribute(self, name, value):
        """
        Add attribute information (other than filenames) to a collection.

        Does not replace existing attribute values.

        Note that the arbitrary information associated with a collection
        may apply either to the collection itself or to all the filenames in
        the collection depending on the meaning the caller wants to
        associate with a particular attribute.

        **Arguments**

            - *name* A string containing the name of the attribute
            to be added.

            - *value* A string containing the value of the attribute
            to add.
            
        **Raises**

            A CollectionException is thrown if unable to add
            the attribute.
        """
        ret, err = replicaCatalogc.collection_add_attribute(self._handle,
                                                            name, value)
        if not ret == 0:
            ex = CollectionException(ex)
            raise ex
        return
    
    def delete_filenames(self, filenames):
        """
        Delete filenames from a collection.

        If one or more of the given filenames is not associated
        with the collection then no deletions are done.

        There must be no correspondingly named logical files or
        corresponding filenames in any of the locations in the collection.

        **Arguments**

            - *filenames* A list of string filenames to delete.

        **Raises**

            A CollectionException is thrown if unable to delete
            the filenames.
        """
        ret, err = replicaCatalogc.collection_delete_filenames(self._handle,
                                                            filenames)
        if not ret == 0:
            ex = CollectionException(ex)
            raise ex
        return

    def delete_attribute(self, name, value):
        """
        Delete attribute information (other than filenames) from a collection.

        **Arguments**

            - *name* A string containing the name of the attribute
            to be deleted.

            - *value* A string containing the value of the attribute
            to delete.
            
        **Raises**

            A CollectionException is thrown if unable to delete
            the attribute.
        """
        ret, err = replicaCatalogc.collection_delete_attribute(self._handle,
                                                            name, value)
        if not ret == 0:
            ex = CollectionException(ex)
            raise ex
        return

    def list_attributes(self, attrNames=None):
        """
        List attribute information (other then filenames) of a collection.

        For each collection only return attribute information for the given attribute names.

        **Arguments**

            - *attrNames* A list of names of the attributes to return. If None, all
            attributes are returned.
        
        **Returns**

            An AttributeSet object is returned containing the attributes

        **Raises*

            A CollectionException is thrown if unable to return the
            attribute list.
        """
        ret, set = replicaCatalogc.collection_list_attributes(self._handle, attrNames)
        if not ret == 0:
            ex = CollectionException(set)
            raise ex
        return AttributeSet.AttributeSet(set)

    def find_locations(self, filenames, all, attrNames=None):
        """
        Search for locations with all (or any) of the given filenames.

        Only return attribute information for the given attribute names.

        **Arguments**

            - *filenames* List of string filenames to return.

            - *all* whether to return locations that contain ALL the given
            filenames (true) or otherwise return locations that contain
            ANY of the given filenames (false)

            - *attrNames* A list of names of the attributes to return. If None, all
            attributes are returned.

        **Returns**

            An EntrySet object is returned containing the locations.

        **Raises**

            A CollectionException is thrown if unable to return the
            locations.
        """
        ret, set = replicaCatalogc.collection_find_locations(self._handle,
                                                             filenames, all, attrNames)
        if not ret == 0:
            ex = CollectionException(set)
            raise ex
        return EntrySet.EntrySet(set)

    def list_locations(self, attrNames=None):
        """
        List the locations in a collection.

        For each location only return attribute information
        for the given attribute name

        **Arguments**

            - *attrNames* A list of names of the attributes to return. If None, all
            attributes are returned.

        
        **Returns**

            An EntrySet object is returned containing the locations.

        **Raises**

            A CollectionException is thrown if unable to return the
            locations.
        """
        ret, set = replicaCatalogc.collection_list_locations(self._handle, attrNames)
        if not ret == 0:
            ex = CollectionException(set)
            raise ex
        return EntrySet.EntrySet(set)

    def list_logicalfiles(self, attrNames=None):
        """
        List the logicalfiles in a collection.

        For each location only return attribute information
        for the given attribute name

        **Arguments**

            - *attrNames* A list of names of the attributes to return. If None, all
            attributes are returned.

        
        **Returns**

            An EntrySet object is returned containing the logicalfiles.

        **Raises**

            A CollectionException is thrown if unable to return the
            logicalfiles.
        """
        ret, set = replicaCatalogc.collection_list_logicalfiles(self._handle, attrNames)
        if not ret == 0:
            ex = CollectionException(set)
            raise ex
        return EntrySet.EntrySet(set)

    def location_create(self, location, url, fileNames=None,
                        objectclass=replicaCatalogc.LOCATION_OBJECTCLASS):
        """
        Create a location.

        Create a location in a collection and associate it with both a given
        URL accessor of a corresponding physical collection and a given
        set of filenames. All the filenames must already exist in the collection,
        and the location itself must not already exist.

        It's assumed a corresponding physical replica has already been
        created and is accessible through the concatenation of the given
        URL accessor with each given filename (which may include a path).
        This implementation does not check that this is the case.

        **Arguments**

            - *location* String name of the location to create.

            - *url* String URL accessor of the corresponding
            physical replica.

            - *filenames* A list of string filenames to add. May be None.

            - *objectclass* objectclass name of the location to be created.
            Must be a subclass of LOCATION_OBJECTCLASS

        **Raises**

            A CollectionException is thrown if unable to create the location.
        """
        ret, err = replicaCatalogc.location_create(self._handle, location,
                                                   url, fileNames, objectclass)
        if not ret == 0:
            ex = CollectionException(err)
            raise ex
        return

    def location_add_filenames(self, location, filenames, add=0):
        """
        Add filenames to a location.

        If some of the given filenames already exist in the location,
        and add is add is true, then add those that don't. Otherwise,
        fail if some of the given filenames already exist and add is
        false. All the filenames must already exist in the collection.

         It's assumed that the corresponding physical files have
         already been created and are accessible through the
         concatenation of the location's URL accessor with each
         given filename (which may include a path). This implementation
         does not check that this is the case.

        **Arguments**

            - *filenames* A list of string filenames to add.

            - *location* name of the existing location to add filenames to.

            - *add*     whether to add given filenames that
            don't exist in the location even if there are some
            given filenames that do exist

        **Raises**

            A CollectionException is thrown if unable to add
            the filenames.
        """
        ret, err = replicaCatalogc.location_add_filenames(self._handle,
                                                            location, filenames, add)
        if not ret == 0:
            ex = CollectionException(err)
            raise ex
        return

    def location_add_attribute(self, location, name, value):
        """
        Add attribute information (other than filenames) to a location.

        Does not replace existing attribute values.

        Note that the arbitrary information associated with a location
        may apply either to the location itself or to all the filenames in
        the location depending on the meaning the caller wants to
        associate with a particular attribute.

        **Arguments**

            - *location* String location name of the existing location to
            add attribute information to.
            
            - *name* A string containing the name of the attribute
            to be added.

            - *value* A string containing the value of the attribute
            to add.
            
        **Raises**

            A CollectionException is thrown if unable to add
            the attribute.
        """
        ret, err = replicaCatalogc.location_add_attribute(self._handle,
                                                          location, name, value)
        if not ret == 0:
            ex = CollectionException(err)
            raise ex
        return

    def location_delete(self, location):
        """
        Delete a location

        **Arguments**

            - *location* String containg the location name to be
            deleted.

        **Raises**

            A CollectionException is thrown if unable to delete the
            location.
        """
        ret, err = replicaCatalogc.location_delete(self._handle, location)
        if not ret == 0:
            ex = CollectionException(err)
            raise ex
        return
    
    def location_delete_filenames(self, location, filenames, delete=0):
        """
        Delete filenames from a location.

        If one or more of the given filenames is not associated
        with the location then no deletions are done.

        **Arguments**

        - *location* A string containing the location to delete
        filenames from.

        - *filenames* A list of string filenames to delete.

        **Raises**

            A CollectionException is thrown if unable to delete
            the filenames.
        """
        ret, err = replicaCatalogc.location_delete_filenames(self._handle,
                                                             location, filenames)
        if not ret == 0:
            ex = CollectionException(err)
            raise ex
        return

    def location_delete_attribute(self, location, name, value):
        """
        Delete attribute information (other than filenames) from a location.

        **Arguments**

            - *location* A string containing the location to delete the
            attribute from.
            
            - *name* A string containing the name of the attribute
            to be deleted.

            - *value* A string containing the value of the attribute
            to delete.
            
        **Raises**

            A CollectionException is thrown if unable to delete
            the attribute.
        """
        ret, err = replicaCatalogc.location_delete_attribute(self._handle,
                                                             location, name, value)
        if not ret == 0:
            ex = CollectionException(err)
            raise ex
        return
    
    def location_list_filenames(self, location):
        """
        List the filenames in a location.

        **Arguments**

            - *location* A string containing the location to list the filenames
            of.

        **Returns**

            A list of strings containing the names of the filenames in the
            location.

        **Raises**

            A CollectionException is thrown if unable to list the filenames.
        """
        ret, names = replicaCatalogc.location_list_filenames(self._handle,
                                                             location)
        if not ret == 0:
            ex = CollectionException(names)
            raise ex
        return names

    def logicalfile_create(self, name, size,
                           objectclass=replicaCatalogc.LOGICALFILE_OBJECTCLASS):
        """
        Create a logical file in a collection.

        The corresponding collection filename must exist. The logical file must not already exist.

        **Arguments**

            - *name* String name of the logicalfile to create.

            - *size* Size in kilobytes of the logical file to add.

            - *objectclass* objectclass name of the logicalfile to be created.
            Must be a subclass of LOGICALFILE_OBJECTCLASS

        **Raises**

            A CollectionException is thrown if unable to create the logicalfile.
        """
        ret, err = replicaCatalogc.logicalfile_create(self._handle, name,
                                                      size, objectclass)
        if not ret == 0:
            ex = CollectionException(err)
            raise ex
        return
    
    def logicalfile_add_attribute(self, logicalfile, name, value):
        """
        Add attribute information (other than filenames) to a logicalfile.

        Does not replace existing attribute values.

        **Arguments**

            - *logicalfile* String name of the logicalfile to
            add attribute information to.
            
            - *name* A string containing the name of the attribute
            to be added.

            - *value* A string containing the value of the attribute
            to add.
            
        **Raises**

            A CollectionException is thrown if unable to add
            the attribute.
        """
        ret, err = replicaCatalogc.logicalfile_add_attribute(self._handle,
                                                          logicalfile, name, value)
        if not ret == 0:
            ex = CollectionException(err)
            raise ex
        return

    def logicalfile_delete(self, logicalfile):
        """
        Delete a logicalfile

        **Arguments**

            - *logicalfile* String containg the logicalfile name to be
            deleted.

        **Raises**

            A CollectionException is thrown if unable to delete the
            logicalfile.
        """
        ret, err = replicaCatalogc.logicalfile_delete(self._handle, logicalfile)
        if not ret == 0:
            ex = CollectionException(err)
            raise ex
        return

    def logicalfile_delete_attribute(self, logicalfile, name, value):
        """
        Delete attribute information from a logicalfile.

        **Arguments**

            - *logicalfile* A string containing the logicalfile to delete the
            attribute from.
            
            - *name* A string containing the name of the attribute
            to be deleted.

            - *value* A string containing the value of the attribute
            to delete.
            
        **Raises**

            A CollectionException is thrown if unable to delete
            the attribute.
        """
        ret, err = replicaCatalogc.logicalfile_delete_attribute(self._handle,
                                                             logicalfile, name, value)
        if not ret == 0:
            ex = CollectionException(err)
            raise ex
        return

    def logicalfile_list_attributes(self, logicalfile, attrNames=None):
        """
        List attribute information (other then filenames) of a collection.

        For each collection only return attribute information for the given attribute names.

        **Arguments**

            - *logicalfile* String name of the logical file to get the attributes from.
            
            - *attrNames* A list of names of the attributes to return. If None, all
            attributes are returned.
        
        **Returns**

            An AttributeSet object is returned containing the attributes

        **Raises*

            A CollectionException is thrown if unable to return the
            attribute list.
        """
        ret, set = replicaCatalogc.logicalfile_list_attributes(self._handle,
                                                               logicalfile, attrNames)
        if not ret == 0:
            ex = CollectionException(set)
            raise ex
        return AttributeSet.AttributeSet(set)
    
class CollectionAttr:
    """
    The main interface to the replica catatlog collection attribute module.

    t provides get and set methods for all of the various handle
    specific attributes, and hides the underlying memory management.
    """
    
    def __init__(self, handle=None):
        """
        Construct an instance.
        
        This will activate the globus_replica_catalog_module, and create
        the collection attr handle..

        **Arguments**

            - *handle* A SWIG'ized pointer to a globus_replica_catalog_collection_handleattr.
        
        **Raises:**
        
            A CollectionAttrException is thrown if unable to activate the module, or
            create the attribute.
        """
        self._activated = 0
        self._handle = None
        ret = utilc.globus_module_activate(replicaCatalogc.get_module())
        if not ret == 0:
            ex = CollectionAttrException("Unable to initialize the module")
            raise ex
        self._activated = 1
        if handle == None:
            ret, handle = replicaCatalogc.handleattr_init()
            if not ret == 0:
                ex = CollectionAttrException(handle)
                raise ex
        self._handle = handle
        return

    def __del__(self):
        """
        Destroys an instance.

        It will close the connection, and deactivate the globus_replica_catalog
        module, and delete the handle.

        **Raises:**
        
            A CollectionAttrException is thrown if unable to destroy the module. This
            exception can't be caught, and should never happend.  It's there mainly
            for debugging if it does happen. :-)
        """
        if not self._handle == None:
            ret, err = replicaCatalogc.handleattr_destroy(self._handle)
            if not ret == 0:
                ex = CollectionAttrException(err)
                raise ex
        ret = utilc.globus_module_deactivate(replicaCatalogc.get_module())
        if not ret == 0:
            ex = CollectionAttrException("Unable to deactivate the module")
            raise ex
        return

    def set_authentication_mode(self, mode, usr=None, passwd=None):
        """
        Set the authentication mode of the connection attributes.

        The current implementation requires the authentication mode be set to
        GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_CLEARTEXT.

        To connect to a replica catalog that allows unauthenticated queries
        (generally not updates as well), set the authentication mode to
        GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_CLEARTEXT
        and pass in None for both username and password.

        **Arguments**

            - *mode* Type of authentication to use when connecting to
            the catalog. One of:
            GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_CLEARTEXT,
            GLOBUS_REPLICA_CATALOG_AUTHENTICATION_MODE_SECURE

            - *usr* The string user name to connectin with if set to auth mode cleartext.

            - * passwd* The string password to use if auth mode is set to cleartext.

        **Raises**

            A CollectionAttrException is thrown if unable to set the authentication
            mode.
        """
        ret, err = replicaCatalogc.set_authentication_mode(self._handle,
                                                           mode, usr, passwd)
        if not ret == 0:
            ex = CollectionAttrException(err)
            raise err
        return

    def get_authentication_mode(self):
        """
        Get the authentication mode of the connection attributes.

        **Returns**

            A 3-tuple is returned. (mode, usr, passwd). If mode is secure,
            usr and passwd are None. The possible modes are the same
            as set_authentication_mode.

        **Raises**

            A CollectionAttrException is thrown if unable to get the
            authentication mode.
        """
        ret, mode, usr, passwd = replicaManagementc.get_authentication_mode(self._handle)
        if not ret == 0:
            ex = CollectionAttrException(mode)
            raise ex
        return (mode, usr, passwd)

    def get_handle(self):
        """
        Return the underlying replica catalog handle attr handle.

        **XXX** This handle is only valid as long as this object exists.  When
        the object is destroyed, the memory this handle points to is freed.

        **Returns:**

            A string containing the pointer.
        """
        return self._handle

class EntrySet:
    """
    The main interface to the replica catatlog entry set module.

    Manages access to the entry set, and hides the underlying memory
    management.
    """
    
    def __init__(self, handle):
        """
        Construct an instance.
        
        This will construct an instance from a given handle. This
        constructor. is for module use, and should not be called. Instead
        calls like Collection.list_collections() returns an entry set instance.

        **Arguments**

            - *handle* A SWIG'ized pointer to a globus_replica_catalog_collection_entry_set.
        """
        self._handle = handle
        return

    def __del__(self):
        """
        Destroys an instance.

        It will destroy the entry set, and free the underlying memory.

        **Raises:**
        
            A EntrySetException is thrown if unable to destroy the entry set. This
            exception can't be caught, and should never happend.  It's there mainly
            for debugging if it does happen. :-)
        """
        if not self._handle == None:
            ret, err = replicaCatalogc.entry_set_destroy(self._handle)
            if not ret == 0:
                ex = EntrySetException(err)
                raise ex
        return

    def __len__(self):
        """
        Returns the length of the set.
        """
        return self.count()

    def __getitem__(self, index):
        """
        Limited sequence operator.

        Allows normal python list iteration. Returns the attribute set
        associated with each entry.
        """
        if index == 0:
            self.first()
        if not self.more():
            raise IndexException
        attrs = self.get_attributes()
        self.next()
        return attrs
    
    def get_attributes(self):
        """
        Return the attribute set of the current entry in the set.

        Must have been preceded by a call to get_first().

        **Retuns**

            An AttributeSet object containg the attributes is returned.

        **Raises**

            An EntrySetException is thrown if unable to return
            the AttributeSet.
        """
        ret, set = replicaCatalogc.entry_set_get_attributes(self._handle)
        if not ret == 0:
            ex = EntrySetException(set)
            raise ex
        return AttributeSet.AttributeSet(set)

    def first(self):
        """
        Set the current entry to the first entry in the set.

        **Raises**

            An EntrySetException is thrown if unable to reset the current
            entry.
        """
        ret, err = replicaCatalogc.entry_set_first(self._handle)
        if not ret == 0:
            ex = EntrySetException(err)
            raise ex
        return

    def next(self):
        """
        Set the current entry to the next entry in the set.

        Must be preceded by a call to first.
        
        **Raises**

            An EntrySetException is thrown if unable to reset the current
            entry.
        """
        ret, err = replicaCatalogc.entry_set_next(self._handle)
        if not ret == 0:
            ex = EntrySetException(err)
            raise ex
        return

    def more(self):
        """
        Whether there are more entries in an entry set, inclusive of the current entry.

        Must be preceded by a call to first.

        **Returns**

            Returns true if there are more entries, and false if none.

        **Raises**

            An EntrySetException is thrown if an error occurs.
        """
        isMore = replicaCatalogc.globus_replica_catalog_entry_set_more(self._handle)
        return isMore

    def count(self):
        """
        Get the number of entries in the entry set.

        **Returns**

            The number of entries is returned.

        **Raises**

            An EntrySetException is thrown if unable to return the number
            of entries.
        """
        ret, count = replicaCatalogc.entry_set_count(self._handle)
        if not ret == 0:
            ex = EntrySetException(count)
            raise ex
        return count

    def get_dn(self):
        """
        Get the DN of the current entry in an entry set.

        Must be preceded by a call to first.

        **Returns**

            A string containing the DN of the current entry is returned.

        **Raises**

            An EntrySetException is thrown if unable to return the DN.
        """
        ret, dn = replicaCatalogc.entry_set_get_dn(self._handle)
        if not ret == 0:
            ex = EntrySetException(dn)
            raise ex
        return dn

    def get_url(self):
        """
        Get the URL of the current entry in an entry set.

        Must be preceded by a call to first.

        **Returns**

            A string containing the url of the current entry is returned.

        **Raises**

            An EntrySetException is thrown if unable to return the url.
        """
        ret, url = replicaCatalogc.entry_set_get_url(self._handle)
        if not ret == 0:
            ex = EntrySetException(url)
            raise ex
        return url

    def get_name(self):
        """
        Get the name of the current entry in an entry set.

        Must be preceded by a call to first.

        **Retuns**

            A string is returned containing the name of the current
            entry.

        **Raises**

            An EntrySetException is thrown if unable to get the name.
        """
        ret, name = replicaCatalogc.entry_set_get_name(self._handle)
        if not ret == 0:
            ex = EntrySetException(name)
            raise ex
        return name
    
    def get_handle(self):
        """
        Return the underlying replica catalog entry set handle.

        **XXX** This handle is only valid as long as this object exists.  When
        the object is destroyed, the memory this handle points to is freed.

        **Returns:**

            A string containing the pointer.
        """
        return self._handle
