############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
from pyGlobus import gsic 
from pyGlobus.util import GlobusException
"""
A module to provide some of the shared GSI functionality.
"""
#########################################################################
# Module functions
########################################################################

def import_external(handle):
    major, minor, cred, time = gsic.import_cred(gsic.GSS_EXPORT_CRED_EXTERNAL,
                                                handle, 0)
    if not major == gsic.GSS_S_COMPLETE:
        list = gsic.display_status(major, minor)
	ex = apply(GSSContextException, list)
	raise ex
    return GSSCred(cred)

def gss_no_cred_factory():
    """
    Returns a GSSCred containing GSS_C_NO_CRED.

    GSS_C_NO_CRED is used by many GSI calls to indicate the usage of the default credential.
    """
    return GSSCred(gsic.get_no_cred_const_ptr())


def gss_init_cred_factory():
    """
    Returns a GSSCred containing an initialized pcd structure.

    Not supported for gt-2.2
    """
    return GSSCred(gsic.get_initialized_gss_cred())


def genreq(issuer_gsscred, sub_gsscred, X509_request, private_key, bits=512):
    """
    Creates a X509_REQ and EVP_PKEY object, gsscred issuer's name is used
    for generation of gsscred sub's name.  gsscred sub object used for ??
    
    Not supported for gt-2.2
    """
    ret = gsic.genreq(issuer_gsscred._handle,
                           sub_gsscred._handle,
                           X509_request._handle,
                           private_key._handle, bits)
    if ret < 0:
        raise GSIException, 'generate request failed'
    return
def release_gpi(cred):
    """function that releases the user proxy created by
    grid_proxy_init function

    cred is a SWIG'ized gss_cred_id_desc pointer

    This is necessary because we can't use this pointer in
    GSSCred object :(
    """
    gsic.free_gpi(cred)

def grid_proxy_init(verbose=1):
    """Returns a SWIG'ized gss_cred_id_desc pointer"""
    ret, cred = gsic.grid_proxy_init(verbose)
    if ret != 0:
        raise GSIException(cred)

    return cred


def sign_request(icred, scred, req):
    """
       Not supported for gt-2.2
       """
    print "icred: ", icred._handle
    print "scred: ", scred._handle
    print "req:   ", req._handle
    return gsic.sign_request(icred._handle, scred._handle, req._handle)
                                 
def get_default_user_credential():
    c = gsic.get_no_cred_const_ptr()
    maj, min = gsic.assist_acquire_cred(0, c)
    if maj == 0:
        return GSSCred(c)
    else:
        return None
    
###########################################################################
# Module Classes
##########################################################################

class GSIException(GlobusException):
    """
    A sub-class of GlobusExcetpion.

    Used as a base class for all other security exceptions.

    **Extends:**

        *pyGlobus.util.GlobusException*
    """
    pass

class GSSContextException(GSIException):
    """
    A sub-class of GSSContextException.

    Used to indicate errors in the GSSContext module.

    **Extends:**

        *pyGlobus.security.GSI.GSIException*
    """
    pass

class GSSNameException(GSIException):
    """
    A sub-class of GSSContextException.

    Used to indicate errors in the GSSName module.

    **Extends:**

        *pyGlobus.security.GSI.GSIException*
    """
    pass

class GSSCredException(GSIException):
    """
    A sub-class of GSIException.

    Used to indicate errors in the GSSCred module.

    **Extends:**

        *pyGlobus.security.GSI.GSIException*
    """
    pass

class GSSContext:
    """
    A proxy class for the underlying gss_ctx_id_t object.
    """

    def __init__(self, handle=None, free=1):
        """
        Constructs an instance.

        Each instance wraps a gss_ctx_id_t object.

        **Arguments:**

            - *handle* An optional argument that, if present, should
            contain a SWIG'ized pointer to a gss_ctx_id_t object.

            - *free* Should the underlying gss_ctx_id_t object be
            freed when an instance is destroyed.
        """
        if handle == None:
            self._handle = gsic.create_context()
        else:
            self._handle = handle
        self._free = free
        return

    def __del__(self):
        """
        Destroys an instance.

        Deletes the underlying gss_ctx_id_t object, and free's the
        memory if the instance owns the underlying handle.
        """
        if self._free:
            major, minor = gsic.delete_context(self._handle)
            if not major == gsic.GSS_S_COMPLETE:
		list = gsic.display_status(major, minor)
		ex = apply(GSSContextException, list)
		raise ex
        return

    def inquire(self):
        """
        Obtain information about the context.

        **Raises:**

            A GSSContextException is raised if an error occurs in the
            underlying gss-api call.

        **Returns:**

            A 7-tuple containing:
            
                - A GSSName object containing the initiators name.
                
                - A GSSName object with the acceptors name.
                
                - An int containing the number of seconds the context
                will remain valid for.
                
                - A string containing a SWIG'ized pointer to a static
                gss_OID struct containing the mechanism type.
                
                - An int containing the context flags.
                
                - A bool indicating if the connection was initiated
                locally.
                
                - A bool indicating if the connection is currently open.
        """
        tuple = gsic.inquire_context(self._handle)
        if not tuple[0] == gsic.GSS_S_COMPLETE:
	    list = gsic.display_status(tuple[0], tuple[1])
	    ex = apply(GSSContextException, list)
	    raise ex
        return GSSName(tuple[2]), GSSName(tuple[3]), tuple[4], \
	       tuple[5], tuple[6], tuple[7], tuple[8]

    def init_context(self, init_cred, target_name, mech_type,req_flags, time_req,input_token):
        """
              Tries to establish a secuirty context with a server

              *Arguements*
              init_cred is a GSSCred object which identifies the client

              target_name is a GSSName object which is the name of the server

              mech_type

              req_flags

              time_req

              input_token a GSSBuffer object
              """
        major, minor, token_string = gsic.init_sec_context(self._handle,init_cred,target_name,mech_type,
                                            req_flags, time_req,input_token)

        return token_string

    def accept_context(self,acceptor_cred_handle, input_token, input_chan_bindings, src_name,
                       mech_type, output_token, delegated_cred_handle):

        major, minor = gsic.accept_sec_context(self._handle, acceptor_cred_handle,input_chan_bindings ,
                                               src_name,mech_type, output_token, delegated_cred_handle)

        return major,minor
        
    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed if
    # self._free is true.
    #######################################################################
    def get_handle(self):
        return self._handle
    

class GSSCred:
    """
    A proxy class for the underlying gss_cred_id_t object.
    """

    def __init__(self, handle=None, free=1):
        """
        Constructs an instance.

        Each instance wraps a gss_cred_id_t object.

        **Arguments:**

            - *handle* A SWIG'ized pointer to a gss_cred_id_t object.

            - *free* Should the underlying gss_cred_id_t object be
            freed when an instance is destroyed.
            """
        if handle==None:
            self._handle = gsic.get_no_cred_const_ptr()
        else:
            self._handle = handle
        self._free = free
        return

    def __del__(self):
        """
        Destroys an instance.

        Deletes the underlying gss_cred_id_t object, and free's the
        memory if the instance owns the underlying handle.
        """
        if self._free:
            major, minor = gsic.release_cred(self._handle)
            if not major == gsic.GSS_S_COMPLETE:
		list = gsic.display_status(major, minor)
		ex = apply(GSSContextException, list)
		raise ex
        return

    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed if
    # self._free is true.
    #######################################################################
    def get_handle(self):
        return self._handle

    def get_name(self):
        major, minor, name = gsic.get_name(self._handle)
        if not major == gsic.GSS_S_COMPLETE:
            list = gsic.display_status(major, minor)
            ex = apply(Exception, list)
            raise ex
        return name

    def get_cred_path(self):
        """
        Not supported for gt-2.2
              """
        if not self._handle:
            raise GSSCredException, 'credential not initialized'
        path = gsic.get_cred_path(self._handle)
        return path

    def get_pcd_certdir(self):
        """
              Not supported for gt-2.2
              """
        if not self._handle:
            raise GSSCredException, 'credential not initialized'
        return gsic.get_pcd_certdir(self._handle)
    
    def set_pcd_certdir(self, certdir):
        """
              Not supported for gt-2.2
              """
        if not self._handle:
            raise GSSCredException, 'credential not initialized'
        gsic.set_pcd_certdir(self._handle, certdir)
        return
    
    def set_pcd_pkey(self, pkey):
        """
              Not supported for gt-2.2
              """
        if not self._handle:
            raise GSSCredException, 'credential not initialized'
        if not pkey._handle:
            raise GSSCredException, 'parameter private key not initialized'
        gsic.set_pcd_upkey(self._handle, pkey._handle)
        return

    def set_cred_globuid(self):
        """
              Not supported for gt-2.2
              """
        if gsic.set_cred_globusuid(self._handle) < 0:
            raise GSSCredException, 'failed to set cred globusuid'
        return

    def verify(self):
        """
              Not supported for gt-2.2
              """
        if not self._handle:
            raise GSSCredException, 'credential not initialized'
        if gsic.verify_pcd(self._handle) < 0:
            raise GSSCredException, 'credential verify failed'
        return
        
    def export_external(self):
        tuple = gsic.export_cred(self._handle, gsic.GSS_EXPORT_CRED_EXTERNAL)
        if not tuple[0] == gsic.GSS_S_COMPLETE:
            list = gsic.display_status(tuple[0], tuple[1])
	    ex = apply(GSSContextException, list)
	    raise ex
        return tuple[2]
    
    def inquire_cred(self):
        major, minor, life, name = gsic.inquire_cred(self._handle)
        if not major == gsic.GSS_S_COMPLETE:
            list = gsic.display_status(major, minor)
            ex = apply(Exception, list)
            raise ex
        return (life, name)


class X509_Request:
    """
       A proxy class for the underlying X509_REQ* objects.

    
       Not supported for gt-2.2
              
       """

    def __init__(self, handle=None, free=1):
        """
        Constructs an instance.

        Each instance wraps X509_REQ* objects.

        **Arguments:**

            - *req_handle* A SWIG'ized pointer to a X509_REQ* object.
            
            - *free* Should the underlying objects be
            freed when an instance is destroyed.
        """
        if handle:
            self._handle = handle
        else:
            self._handle = gsic.malloc_X509_REQ()
        self._free = free
        return

    def __del__(self):
        """
        Destroys an instance.

        Deletes the underlying gss_cred_id_t object, and free's the
        memory if the instance owns the underlying handle.
        """
        
        if self._free:
            gsic.free_X509(self._handle)
        return
    
class EVP_Key:
    """
       A proxy class for the underlying X509_REQ* EVP_PKEY* objects.
       Not supported for gt-2.2         
        """

    def __init__(self, handle=None, free=1):
        """
        Constructs an instance.

        Each instance wraps X509_REQ* EVP_PKEY* objects.

        **Arguments:**

            - *req_handle* A SWIG'ized pointer to a X509_REQ* object.

            - *key_handle* A SWIG'ized pointer to a EVP_PKEY* object.
            
            - *free* Should the underlying objects be
            freed when an instance is destroyed.
        """
        if handle:
            self._handle = handle
        else:
            self._handle = gsic.malloc_EVP_PKEY()
        self._free = free
        return
    def __del__(self):
        """
        Destroys an instance.

        Deletes the underlying gss_cred_id_t object, and free's the
        memory if the instance owns the underlying handle.
        """
        
        if self._free:
            gsic.free_EVP_PKEY(self._handle)
        return
"""
def generate_request(icred, bits=512):
    tup = gsic.genreq(icred._handle, bits)
    if tup == None:
        raise RuntimeError, 'security.generate_request: genreq failed'
    return tup
"""     
             
class GSSName:
    """
    A proxy class for the underlying gss_name_t object.
    """

    def __init__(self, handle=None, free=1):
        """
        Constructs an instance.

        Each instance wraps a gss_name_t object.

        **Arguments:**

            - *handle* An optional argument that, if present, should
            contain a SWIG'ized pointer to a gss_name_t object.

            - *free* Should the underlying gss_name_t object be
            freed when an instance is destroyed.
        """
        self._handle = handle
        self._free = free
        return

    def __del__(self):
        """
        Destroys an instance.

        Deletes the underlying gss_name_t object, and free's the
        memory if the instance owns the underlying handle.

        **Raises:**

            A GSSNameException is raised if an error occurs
            in the underlying gss-api call.
        """
        if not self._handle == None:
            if self._free:
                major, minor = gsic.release_name(self._handle)
            if not major == gsic.GSS_S_COMPLETE:
                list = gsic.display_status(major, minor)
		ex = apply(GSSContextException, list)
		raise ex
        return

    def display(self):
        """
        Returns the name as a string.

        **Raises:**

            A GSSNameException is raised if an error occurs
            in the underlying gss-api call.

        **Returns:**

            A string containing the name.
        """
        major, minor, name = gsic.display_name(self._handle)
        if not major == gsic.GSS_S_COMPLETE:
            print "display: ", str((major, minor))
            list = gsic.display_status(major, minor)
	    ex = apply(GSSContextException, list)
	    raise ex
        return name

    ########################################################################
    # XXX This handle is only valid as long as this object exists.  When
    # the object is destroyed, the memory this handle points to is freed if
    # self._free is true.
    #######################################################################
    def get_handle(self):
        return self._handle
