############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
"""
A module containing utility functions, e.g. buffers, module activation, etc.
"""

from pyGlobus import utilc
import exceptions

class GlobusException(exceptions.Exception):
    """
    A simple exception class.

    Used as a super-class for all exceptions used in pyGlobus. This will
    allow simpler exception handling.

    **Extends:**
    
        *exceptions.Exception* The baseclass for all exceptions in python.
    """
    pass

class GlobusBuffferException(GlobusException):
    """
    A simple subclass of GlobusException.

    Handles errors in the buffer class.
    """
    pass

class Off_t:
    """
    A wrapper class for globus_off_t pointers

    """
    def __init__(self, handle):
        self._handle = handle

    def __del__(self):
        """ frees the memory to the underlying handle
        """
        utilc.free_off_t(self._handle)

    def getLong(self):
        """ Returns a python long representation of the
        underlying off_t pointer """
        return utilc.convert_off_t(self._handle)

class Abstime:
    """
        A wrapper class for the globus_abstime_t type
        Handles the memory and allows some access to the
        underlying timeval struct
        """

    def __init__(self, handle):
        self._handle = handle

    def __del__(self):
        """ frees the memory associated with the abs_time_t handle"""
        utilc.free_abstime(self._handle)

    def getSecs(self):
        """Returns a long of the underlying seconds of the underlying abs_time_t
        handle """
        return utilc.getSecs(self._handle)

    def getNSecs(self):
        """Returns a long of the underlying u_seconds of the underlying abs_time_t
        handle """
        return utilc.getNSecs(self._handle)

   

    
class Buffer:
    """
    A wrapper class for globus_byte_t arrays.

    Allows the manipulation of globus_byte_t arrays. This class malloc's the
    underlying memory in the constructor, and free's it during destruction. It
    supports returning the underlying memory as a python string. It also supports
    the readline and readlines methods.
    """
    def __init__(self, size):
        """
        Contruct a Buffer object.

        This methods causes underlying memory to be malloc'd.

        **Arguments:**
        
            size The size of the buffer to create.

        **Raises:**
        
            A GlobusBufferException is thrown if unable to create
            the buffer object.
        """
        ret = utilc.globus_module_activate(utilc.get_module())
        if not ret == 0:
            ex = GlobusBufferException("unable to init the common module")
            raise ex
        self._handle, self._size = utilc.create_buffer(size)
        self._str = None
        return

    def __del__(self):
        """
        Destroys the Buffer object.

        Free's the underlying memory.
        """
        utilc.destroy_buffer(self._handle)
        ret = utilc.globus_module_deactivate(utilc.get_module())
        return
   
    
    def __len__(self):
        """
        Get the size of the Buffer object.

        **Returns:**
        
            An int containing the size of the Buffer object.
        """
        return self._size
    
    def get_handle(self):
        """
        Get the underlying pointer to the globus_byte_t array.

        **Returns:**
        
            A SWIG'ized pointer to the underlying byte array.

        **XXX** This handle is only valid as long as this object exists.  When
        the object is destroyed, the memory this handle points to is freed.
        """
        return self._handle
    
    def set_handle(self, handle):
        """ Sets a globus_byte_t array to a Buffer object"""
        self._handle = handle
    
    def get_size(self):
        """
        Get the size of the Buffer object.

        **Returns:**
        
            An int containing the size of the Buffer object.
        """
        return self._size
    
    def as_string(self, size):
        """
        Convert the underlying globus_byte_t array into a Python string.

        **Arguments:**
        
            size The number of bytes to convert.
            
        **Returns:**
        
            A string containg the contents of the Buffer.
        """
        return utilc.buffer_to_string(self._handle, size)

    def readline(self):
        """
        Return a single line as a string.

        **Returns:**
        
            A string containing the next line in the Buffer.
        """
        if self._str == None:
            str = utilc.buffer_to_string(self._handle, self._size)
        else:
            str = self._str
        i = str.find('\n')
        if i >= 0:
            self._str = str[i+1:]
            line = str[:i]
        else:
            self._str = ""
            line = str
        return line

    def readlines(self):
        """
        Return a list containing each line in the Buffer.

        **Returns:**
        
            A list of strings containing each line in the Buffer.
        """
        list = []
        while 1:
            line = self.readline()
            if line == "":
                break
            list.append(line)
            continue
        return list
    
