############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from distutils.core import setup, Extension
from distutils.spawn import spawn
from distutils.sysconfig import get_python_inc
from distutils.sysconfig import get_config_vars
from os import environ, popen4
from os.path import isfile
import sys, string, re, exceptions

GLOBUS_LOCATION = None
FLAVOR = "gcc32dbgpthr"
RUN_SWIG = 0
RUN_HAPPYDOC = 0
GLOBUS_LIBS = None
GLOBUS_PACKAGES = None
SWIG_ARGS = ["-python", "-noproxy", "-DHAVE_CONFIG_H",
             "-I%s" % get_python_inc(plat_specific=1)]
GLOBUS_DEPS = {"util" : "globus_common"}
MAKEFILE_HEADER = None
BUILT_LIST = []
FAILED_LIST = []
MODULE_LIST = ['util','ftpClient','ftpControl','gassCache', 'gassCopy', 'gassFile','gassServerEZ',
               'gassTransfer','gramClient','io','replicaCatalog', 'replicaManagement','security']

# Get GLOBUS_LOCATION, exit if it doesn't exist
try:
    GLOBUS_LOCATION = environ["GLOBUS_LOCATION"]
except KeyError:
    print "GLOBUS_LOCATION must be set before installing pyGlobus"
    sys.exit(-1)

# parse special command line options and remove them
# this way distutils won't attempt to parse them.
args = sys.argv[1:]
for arg in args:
    if string.find(arg, '--flavor=') == 0:
        FLAVOR = string.split(arg, '=')[1]
        sys.argv.remove(arg)
    elif string.find(arg, '--run-swig') == 0:
        RUN_SWIG = 1
        sys.argv.remove(arg)
    elif string.find(arg, '--run-happydoc') == 0:
        RUN_HAPPYDOC = 1
        sys.argv.remove(arg)
    elif string.find(arg, '--with-modules') == 0:
        modules = string.split(arg, '=')[1]
        mList = []
        while not modules == '':
            indx = modules.find(',')
            if indx == -1:
                mName = modules
                modules = ''
            else:
                mName = modules[:indx]
                modules = modules[indx +1:]
            mList.append(mName)
        MODULE_LIST = mList
        sys.argv.remove(arg)
    
        


#Find globus-makefile-header. If unable to find it, exit
if isfile("%s/sbin/globus-makefile-header" % GLOBUS_LOCATION):
    MAKEFILE_HEADER = "%s/sbin/globus-makefile-header -flavor=%s" % (GLOBUS_LOCATION, FLAVOR)
elif isfile("%s/bin/globus-makefile-header" % GLOBUS_LOCATION):
    MAKEFILE_HEADER = "%s/bin/globus-makefile-header -flavor=%s" % (GLOBUS_LOCATION, FLAVOR)
else:
    print "Unable to locate globus-makefile-header in either $GLOBUS_LOCATION/[bin,sbin]. This must be found before installing pyGlobus."
    sys.exit(-2)
    
HAPPYDOC = ["happydoc", "-d", "./doc", "pyGlobus", "formatter_levelOneHeadingBackgroundColor='#CCCCCC'", "formatter_levelOneHeadingForegroundColor='000000'", "formatter_levelTwoHeadingBackgroundColor='#CCCCCC'", "formatter_levelTwoHeadingForegroundColor='000000'", "./pyGlobus"]
errRegEx = ".*ERROR.*" 
incRegEx = r"\s*GLOBUS_INCLUDES\s*=\s*"
cflagsRegEx = r"\s*GLOBUS_CFLAGS\s*=\s*"
pkgLibRegEx = r"\s*GLOBUS_PKG_LIBS\s*=\s*"
ldFlagsRegEx = r"\s*GLOBUS_LDFLAGS\s*=\s*"
libsRegEx = r"\s*GLOBUS_LIBS\s*=\s*"
ERR_PATTERN = re.compile(errRegEx)
INC_PATTERN = re.compile(incRegEx)
CFLAGS_PATTERN = re.compile(cflagsRegEx)
PKGLIBS_PATTERN = re.compile(pkgLibRegEx)
LDFLAGS_PATTERN = re.compile(ldFlagsRegEx)
LIBS_PATTERN = re.compile(libsRegEx)

class DependencyException(exceptions.Exception):
    pass

def build_module_lists(mList = MODULE_LIST):
    print "Checking for which modules to build"
    swigList = []
    pyModList = []
    eModList = []
    for module in mList:
        if module == 'util':
            check_util(pyModList, eModList, swigList)
        elif module == 'ftpClient':
            check_ftpClient(pyModList, eModList, swigList)
        elif module == 'ftpControl':
            check_ftpControl(pyModList, eModList, swigList)
        elif module == 'gassCache':
            check_gassCache(pyModList, eModList, swigList)
        elif module == 'gassCopy':
            check_gassCopy(pyModList, eModList, swigList)
        elif module == 'gassFile':
            check_gassFile(pyModList, eModList, swigList)
        elif module == 'gassServerEZ': 
            check_gassServerEZ(pyModList, eModList, swigList)
        elif module == 'gassTransfer':
            check_gassTransfer(pyModList, eModList, swigList)
        elif module == 'gramClient':
            check_gramClient(pyModList, eModList, swigList)
        elif module == 'io':
            check_io(pyModList, eModList, swigList)
        elif module == 'replicaCatalog':
            check_replicaCatalog(pyModList, eModList, swigList)
        elif module == 'replicaManagement':
            check_replicaManagement(pyModList, eModList, swigList)
        elif module == 'security':
            check_security(pyModList, eModList, swigList)
        else:
            raise DependencyException , "bad name: %s in module_list" %(module)
    return pyModList, eModList, swigList

def check_util(pyModList, eModList, swigList):
    dep = "globus_common"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the util package."
        print "This package is required to build all of pyGlobus."
        print "The following is the error message from globus-makefile-header."
        print ex.args
        print "Now exiting from the build."
        sys.exit(-1)
    e = Extension("utilc", ["src/util_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("util.i", "util_wrap.c"))
    pyModList.append("pyGlobus.util")
    pyModList.append("pyGlobus.test.test_util")
    pyModList.append("pyGlobus.test.test_pyGlobus")
    pyModList.append("pyGlobus.test.config") 
    BUILT_LIST.append("util")
    return

def check_ftpClient(pyModList, eModList, swigList):
    dep = "globus_ftp_client"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the ftpClient package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("ftpClient")
        return
    e = Extension("ftpClientc", ["src/ftpClient_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("ftpClient.i", "ftpClient_wrap.c"))
    pyModList.append("pyGlobus.ftpClient")
    pyModList.append("pyGlobus.test.test_FtpClient")
    pyModList.append("pyGlobus.test.test_ftpClient")
    pyModList.append("pyGlobus.test.test_EasyFtpClient")
    BUILT_LIST.append("ftpClient")
    return

def check_ftpControl(pyModList, eModList, swigList):
    dep = "globus_ftp_control"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the ftpControl package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("ftpControl")
        return
    e = Extension("ftpControlc", ["src/ftpControl_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("ftpControl.i", "ftpControl_wrap.c"))
    pyModList.append("pyGlobus.ftpControl")
    pyModList.append("pyGlobus.test.test_FtpControl")
    pyModList.append("pyGlobus.test.test_ftpControl")
    BUILT_LIST.append("ftpControl")
    return

def check_gassCache(pyModList, eModList, swigList):
    dep = "globus_gass_cache"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
        
    except DependencyException, ex:
        print "Missing dependency to build the gassCache package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("gassCache")
        return
    e = Extension("gassCachec", ["src/gassCache_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("gassCache.i", "gassCache_wrap.c"))
    pyModList.append("pyGlobus.gassCache")
    pyModList.append("pyGlobus.test.test_GassCache")
    pyModList.append("pyGlobus.test.test_gassCache")
    BUILT_LIST.append("gassCache")
    return

def check_gassCopy(pyModList, eModList, swigList):
    dep = "globus_gass_copy"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the gassCopy package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("gassCopy")
        return
    e = Extension("gassCopyc", ["src/gassCopy_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("gassCopy.i", "gassCopy_wrap.c"))
    pyModList.append("pyGlobus.gassCopy")
    pyModList.append("pyGlobus.test.test_GassCopy")
    pyModList.append("pyGlobus.test.test_gassCopy")
    BUILT_LIST.append("gassCopy")
    return

def check_gassFile(pyModList, eModList, swigList):
    dep = "globus_gass_file"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the gassFile package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("gassFile")
        return
    e = Extension("gassFilec", ["src/gassFile_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("gassFile.i", "gassFile_wrap.c"))
    pyModList.append("pyGlobus.gassFile")
    pyModList.append("pyGlobus.test.test_GassFile")
    pyModList.append("pyGlobus.test.test_gassFile")
    BUILT_LIST.append("gassFile")
    return

def check_gassServerEZ(pyModList, eModList, swigList):
    dep = "globus_gass_server_ez"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the gassServerEZ package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("gassServerEZ")
        return
    e = Extension("gassServerEZc", ["src/gassServerEZ_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("gassServerEZ.i", "gassServerEZ_wrap.c"))
    pyModList.append("pyGlobus.gassServerEZ")
    pyModList.append("pyGlobus.test.test_GassServerEZ")
    pyModList.append("pyGlobus.test.test_gassServerEZ")
    BUILT_LIST.append("gassServerEZ")
    return

def check_gassTransfer(pyModList, eModList, swigList):
    dep = "globus_gass_transfer"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the gassTransfer package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("gassTransfer")
        return
    e = Extension("gassTransferc", ["src/gassTransfer_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("gassTransfer.i", "gassTransfer_wrap.c"))
    pyModList.append("pyGlobus.gassTransfer")
    pyModList.append("pyGlobus.test.test_GassTransfer")
    pyModList.append("pyGlobus.test.test_gassTransfer")
    BUILT_LIST.append("gassTransfer")
    return

def check_gramClient(pyModList, eModList, swigList):
    dep = "globus_gram_client"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the gramClient package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("gramClient")
        return
    e = Extension("gramClientc", ["src/gramClient_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("gramClient.i", "gramClient_wrap.c"))
    pyModList.append("pyGlobus.gramClient")
    pyModList.append("pyGlobus.test.test_GramClient")
    pyModList.append("pyGlobus.test.test_gramClient")
    pyModList.append("pyGlobus.test.testGramClient")
    pyModList.append("pyGlobus.test.testGram")
    BUILT_LIST.append("gramClient")
    return

def check_io(pyModList, eModList, swigList):
    dep = "globus_io"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the io package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("io")
        return
    e = Extension("ioc", ["src/io_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("io.i", "io_wrap.c"))
    pyModList.append("pyGlobus.io")
    pyModList.append("pyGlobus.asynGSI")
    pyModList.append("pyGlobus.GSISOAP")
    pyModList.append("pyGlobus.GSIasyncore")
    pyModList.append("pyGlobus.ZSISOAP")
    pyModList.append("pyGlobus.test.test_GSITcpClient")
    pyModList.append("pyGlobus.test.test_io")
    pyModList.append("pyGlobus.test.test_GSITCPSocket")
    pyModList.append("pyGlobus.test.test_GSITCPSocketServer")
    pyModList.append("pyGlobus.test.test_tcp_client")
    pyModList.append("pyGlobus.test.test_tcp_server")
    pyModList.append("pyGlobus.test.test_ZSISoap")
    BUILT_LIST.append("io")
    return

def check_replicaCatalog(pyModList, eModList, swigList):
    dep = "globus_replica_catalog"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the replicaCatalog package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("replicaCatalog")
        return
    e = Extension("replicaCatalogc", ["src/replicaCatalog_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("replicaCatalog.i", "replicaCatalog_wrap.c"))
    pyModList.append("pyGlobus.replicaCatalog")
    pyModList.append("pyGlobus.test.test_Collection")
    pyModList.append("pyGlobus.test.test_replicaCatalog")
    BUILT_LIST.append("replicaCatalog")
    return

def check_replicaManagement(pyModList, eModList, swigList):
    dep = "globus_replica_management"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the replicaManagement package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("replicaManagment")
        return
    e = Extension("replicaManagementc", ["src/replicaManagement_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    swigList.append(("replicaManagement.i", "replicaManagement_wrap.c"))
    pyModList.append("pyGlobus.replicaManagement")
    pyModList.append("pyGlobus.test.test_Session")
    pyModList.append("pyGlobus.test.test_replicaManagement")
    BUILT_LIST.append("replicaManagement")
    return

def check_security(pyModList, eModList, swigList):
    dep = "globus_io"
    try:
        libList, incList, cFlagsStr, ldFlagsList = get_makefile_output(dep)
    except DependencyException, ex:
        print "Missing dependency to build the security package."
        print "Will skip this package"
        print "The following error from globus-makefile-header lists the missing dependencies."
        print ex.args
        FAILED_LIST.append("security")
        return
    
        
    e = Extension("gsic", ["src/gsi_wrap.c"], include_dirs=incList,
                  library_dirs=ldFlagsList, extra_compile_args=[cFlagsStr],
                  libraries=libList)
    eModList.append(e)
    #swigList.append(("sslutils.i", "sslutils_wrap.c")) 
    swigList.append(("gsi.i", "gsi_wrap.c"))
    pyModList.append("pyGlobus.security")
    pyModList.append("pyGlobus.test.test_gpi")
    pyModList.append("pyGlobus.test.test_GSSCred")
    pyModList.append("pyGlobus.test.test_GSSContext")
    pyModList.append("pyGlobus.test.test_GSSName")
    pyModList.append("pyGlobus.test.test_proxy")
    pyModList.append("pyGlobus.test.test_security")
    pyModList.append("pyGlobus.test.test_GridProxyInit") 
    BUILT_LIST.append("security")
    return

def get_makefile_output(dep):
    pLibsStr = None
    eLibsStr = None
    cFlagsStr = None
    includeStr = None
    ldFlagsStr = None
    libList = []
    incList = []
    ldFlagsList = []
    inStrm, outStrm = popen4("%s %s" % (MAKEFILE_HEADER, dep))
    for line in outStrm.readlines():
        m = ERR_PATTERN.match(line)
        if m:
            raise DependencyException(line)
        m = INC_PATTERN.match(line)
        if m:
            includeStr = INC_PATTERN.split(line)[1]
            for str in string.split(includeStr):
                s = string.strip(str)[2:]
                incList.append(s)
            continue
        m = CFLAGS_PATTERN.match(line)
        if m:
            cFlagsStr = CFLAGS_PATTERN.split(line)[1]
            continue
        m = PKGLIBS_PATTERN.match(line)
        if m:
            pLibsStr = PKGLIBS_PATTERN.split(line)[1]
            i = 0
            for str in string.split(pLibsStr):
                s = string.strip(str)[2:]
                libList.insert(i, s)
                i += 1
            continue
        m = LDFLAGS_PATTERN.match(line)
        if m:
            ldFlagsStr = LDFLAGS_PATTERN.split(line)[1]
            for str in string.split(ldFlagsStr):
                s = string.strip(str)[2:]
                ldFlagsList.append(s)
            continue
        m = LIBS_PATTERN.match(line)
        if m:
            eLibsStr = LIBS_PATTERN.split(line)[1]
            for str in string.split(eLibsStr):
                s = string.strip(str)[2:]
                libList.append(s)
            continue
    if cFlagsStr == None:
        raise DependencyException, "cFlagsStr == None: check globus-makefile-header"

    return libList, incList, cFlagsStr, ldFlagsList

# Distutils doesn't currently work with recent 1.3.x
# versions of swig.
def run_swig(swigList):
    for s in swigList:
        print s
        cmd = ["swig"] + SWIG_ARGS + ["-o", "./src/%s" % (s[1]),
                                      "./src/%s" % (s[0])]
        spawn(cmd, verbose=1)

def run_happydoc():
    spawn(HAPPYDOC, verbose=1)
    return

#Remove flags from the compile line that through lots
# of warnings.
cv = get_config_vars()
opts = cv["OPT"]
opts = opts.replace("-Wall", "")
opts = opts.replace("-Wstrict-prototypes", "")
cv["OPT"] = opts

pyModList, eModList, swigList = build_module_lists()
if RUN_SWIG:
    run_swig(swigList)
    

setup(name="pyGlobus", version="gt-2.0-1.0.1", description="Python Globus Toolkit wrapper", author="Keith Jackson", author_email="KRJackson@lbl.gov",
      url="http://www-itg.lbl.gov/gtg/pyGlobus/", 
      py_modules=pyModList, ext_package="pyGlobus", ext_modules=eModList)

if RUN_HAPPYDOC:
    run_happydoc()
    
print "Built the following packages:\n"
for mod in BUILT_LIST:
    print mod
if not FAILED_LIST == []:
    print "\n\nFailed to build the following packages due to missing dependencies:\n"
    for mod in FAILED_LIST:
        print mod

