/*############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################*/
%module gassCachec
%{
#include "globus_gass_cache.h"
%}

typedef int globus_bool_t;
typedef struct globus_gass_cache_entry_s
{
    char *                    url;
    char *                    filename;
    /* modification timestamp (seconds since the epoch) */
    unsigned long             timestamp;
    
    /* new vor version 1.1 
       The tag that has acquired this entry's lock, or GLOBUS_NULL
       if the entry is unlocked */    
    char                      pending;
    char *                    lock_tag;
    /* number of tags in the tag array */
    unsigned long             num_tags;
    /* array of tags for this entry    */
    globus_gass_cache_tag_t*  tags;       

} globus_gass_cache_entry_t;

%include ./src/pyGlobus-typemaps.i

%apply char **OUTPUT { char **file };
%apply unsigned long *ulongRet { unsigned long *timestamp };

%types(globus_gass_cache_entry_t*);

%constant GLOBUS_GASS_CACHE_ADD_NEW = 1;
%constant GLOBUS_GASS_CACHE_URL_NOT_FOUND = 2;
%constant GLOBUS_GASS_CACHE_ADD_EXISTS = 3;

%inline %{
globus_module_descriptor_t* get_module(void) {
    return GLOBUS_GASS_CACHE_MODULE;
}

PyObject*
cache_open(PyObject* pyPath) {
    int ret;
    PyObject *tuple, *ptr;
    char *path;
    globus_gass_cache_t *handle;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "cache_open: PyTuple_New()");
        return NULL;
    }
    handle = (globus_gass_cache_t*)globus_malloc(sizeof(globus_gass_cache_t));
    if (pyPath == Py_None) {
        path = NULL;
    }
    else {
        path = PyString_AsString(pyPath);
    }
    ret = globus_gass_cache_open(path, handle);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)handle, SWIGTYPE_p_globus_gass_cache_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

void
handle_free(globus_gass_cache_t *handle) {
    globus_free(handle);
    return;
}

int
cache_close(globus_gass_cache_t *handle) {
    int ret;
    ret = globus_gass_cache_close(handle);
    return ret;
}

int
cache_add(globus_gass_cache_t *handle,
          char *url, char *tag, globus_bool_t create,
          unsigned long *timestamp, char **file) {
    int ret;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gass_cache_add(handle, url, tag, create, timestamp, file);
    Py_END_ALLOW_THREADS
    return ret;
}

PyObject*
cache_list(globus_gass_cache_t *handle) {
    int ret, size;
    PyObject *tuple, *ptr, *list, *entry, *tags, *tagTuple, *tmp;
    globus_gass_cache_entry_t *entries;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "cache_list: PyTuple_New()");
        return NULL;
    }
    ret = globus_gass_cache_list(handle, &entries, &size);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    else {
        int i;
        unsigned long j;
        list = PyList_New(0);
        for (i=0; i < size; i++) {
            if (!(entry=PyTuple_New(6)))
            {
                PyErr_SetString(PyExc_RuntimeError, 
                                "cache_list: PyTuple_New()");
                return NULL;
            }
            PyTuple_SET_ITEM(entry, 0, Py_BuildValue("s", entries[i].url));
            PyTuple_SET_ITEM(entry, 1, Py_BuildValue("s", entries[i].filename));
            if (sizeof(unsigned long) == 8) {
                tmp = PyLong_FromUnsignedLong(entries[i].timestamp);
            }
            else {
                tmp = PyInt_FromLong(entries[i].timestamp);
            }
            PyTuple_SET_ITEM(entry, 2, tmp);
            PyTuple_SET_ITEM(entry, 3, Py_BuildValue("c", entries[i].pending));
            PyTuple_SET_ITEM(entry, 4, Py_BuildValue("s", entries[i].lock_tag));
            tags = PyList_New(0);
            for (j=0; j < entries[i].num_tags; j++) {
                if (!(tagTuple=PyTuple_New(2)))
                {
                    PyErr_SetString(PyExc_RuntimeError, 
                                    "cache_list: PyTuple_New()");
                    return NULL;
                }
                PyTuple_SET_ITEM(tagTuple, 0, Py_BuildValue("s", entries[i].tags->tag));
                PyTuple_SET_ITEM(tagTuple, 1, Py_BuildValue("i", entries[i].tags->count));
                PyList_Append(tags, tagTuple);
            }
            PyTuple_SET_ITEM(entry, 5, tags);
            PyList_Append(list, entry);
        }
        PyTuple_SET_ITEM(tuple, 1, list);
        globus_gass_cache_list_free(entries, size);
    }
    return tuple;
}

%}

extern 
int
globus_gass_cache_add_done(globus_gass_cache_t *handle, char *url,
                           char *tag, unsigned long timestamp);

extern
int
globus_gass_cache_delete_start(globus_gass_cache_t *handle, char *url,
                               char *tag, unsigned long *timestamp);

extern
int
globus_gass_cache_delete(globus_gass_cache_t *handle, char *url, char *tag,
                         unsigned long timestamp, globus_bool_t isLocked);

extern
int
globus_gass_cache_cleanup_tag(globus_gass_cache_t *handle, 
                              char *url, char *tag);
extern
int
globus_gass_cache_cleanup_file(globus_gass_cache_t *handle, char* url);

extern
const char*
globus_gass_cache_error_string(int err);
